#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr  7 15:29:46 2020

@author: Anders Ringbom; Swedish Defence Research Agency (FOI)
         anders.ringbom@foi.se

This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
import sys
sys.path.append('/home/ringbom/Source_svn/Source/trunk/pyxe/spex/openSpex')

import wx
import os
import copy
import pickle
import locale
import webbrowser
import tempfile
from os import path

from open_spex import utils as ut
from open_spex import menu as menu
from open_spex import BGMeasurement as bg
from open_spex import BGSample as bgs
from open_spex import bgm as bgm
from open_spex import dialogs as di
from open_spex import measurementPanel as sp
from open_spex import helpOpenSpex as h
from open_spex import aboutMessage as ab
from open_spex import xenonSampleSet as xs
from open_spex import optionsPanel as op
from open_spex import plotPanel as pp
from open_spex import version

LICENSE = """
MIT License

Copyright (c) 2021 Swedish Defence Research Agency (FOI)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. """

VERSION = version.version

font = 'Liberation Mono'
if os.name == 'nt':
    font = 'Courier New'

class mainFrame(wx.Frame):
    """
    Main frame for the xenon analysis tool 'openSpex'
    """
    def __init__(self, parent, title):
        wx.Frame.__init__(self, parent, title=title, 
                          size=(1500,900), pos = (100,50))
        locale.setlocale(locale.LC_ALL, '')
        self.version = VERSION
        self.options = op.options()
        self.bgsample = None
        self.sample = None
        self.gasbk = None
        self.detbk = None
        self.qc = None
        self.calib = None
        self.mset = None
        self.tweakedLastSample = False
        self.tweakedLastGasbk = False
        
        #Menu
        self.menu = menu.menu()
        self.SetMenuBar(self.menu)
        self.Bind(wx.EVT_MENU, self.OpenPHD, self.menu.openSample)
        self.Bind(wx.EVT_MENU, self.OpenPHD, self.menu.openGasbk)
        self.Bind(wx.EVT_MENU, self.OpenPHD, self.menu.openDetbk)
        self.Bind(wx.EVT_MENU, self.OpenPHD, self.menu.openQC)
        self.Bind(wx.EVT_MENU, self.OpenPHD, self.menu.openCalib)
        self.Bind(wx.EVT_MENU, self.PrintPHD, self.menu.printSample)
        self.Bind(wx.EVT_MENU, self.PrintPHD, self.menu.printGasbk)
        self.Bind(wx.EVT_MENU, self.PrintPHD, self.menu.printDetbk)
        self.Bind(wx.EVT_MENU, self.PrintPHD, self.menu.printQC)
        self.Bind(wx.EVT_MENU, self.PrintPHD, self.menu.printCalib)
        self.Bind(wx.EVT_MENU, self.OnReadObject, self.menu.openMeasurement)
        self.Bind(wx.EVT_MENU, self.OnReadObject, self.menu.openDataset)
        self.Bind(wx.EVT_MENU, self.OnClearLog, self.menu.clearLog)
        self.Bind(wx.EVT_MENU, self.OnQuit, self.menu.quit)
        self.Bind(wx.EVT_MENU, self.OnSavePHD, self.menu.saveSamplePHD)
        self.Bind(wx.EVT_MENU, self.OnSavePHD, self.menu.saveGasbkPHD)
        self.Bind(wx.EVT_MENU, self.OnSavePHD, self.menu.saveDetbkPHD)
        self.Bind(wx.EVT_MENU, self.OnSavePHD, self.menu.saveQCPHD)
        self.Bind(wx.EVT_MENU, self.OnSavePHD, self.menu.saveCalibPHD)
        self.Bind(wx.EVT_MENU, self.OnSaveReport, self.menu.saveMeasurementReport)
        self.Bind(wx.EVT_MENU, self.OnSaveReport, self.menu.saveMeasurementsetReport)
        self.Bind(wx.EVT_MENU, self.OnSaveObject, self.menu.saveMeasurement)
        self.Bind(wx.EVT_MENU, self.OnSaveObject, self.menu.saveDataset)
        self.Bind(wx.EVT_MENU, self.OnSaveMsetCSV, self.menu.saveDatasetCSV)
        self.Bind(wx.EVT_MENU, self.OnAnalyze, self.menu.analyze)
        self.Bind(wx.EVT_MENU, self.OnUnmarkMset, self.menu.unmarkMset)
        self.Bind(wx.EVT_MENU, self.OnClearMea, self.menu.clearMea)
        self.Bind(wx.EVT_MENU, self.OnCalcMset, self.menu.calcMsetFile)
        self.Bind(wx.EVT_MENU, self.OnReportMset, self.menu.reportMset)
        self.Bind(wx.EVT_MENU, self.OnPlotMset, self.menu.plotMset)
        self.Bind(wx.EVT_MENU, self.OnFreqMset, self.menu.freqMset)
        self.Bind(wx.EVT_MENU, self.OnAppendMset, self.menu.appendMset)
        self.Bind(wx.EVT_MENU, self.OnMircMset, self.menu.mirc3Mset)
        self.Bind(wx.EVT_MENU, self.OnMircMset, self.menu.mirc4Mset)
        self.Bind(wx.EVT_MENU, self.OnPlotParam, self.menu.plotXeVol)
        self.Bind(wx.EVT_MENU, self.OnPlotParam, self.menu.plotXeYield)
        self.Bind(wx.EVT_MENU, self.OnPlotParam, self.menu.plotAirVol)
        self.Bind(wx.EVT_MENU, self.OnPlotParam, self.menu.plotErrors)
        self.Bind(wx.EVT_MENU, self.OnClearMset, self.menu.clearMset)
        self.Bind(wx.EVT_MENU, self.OnOptions, self.menu.options)
        self.Bind(wx.EVT_MENU, self.OnHelp, self.menu.help)
        self.Bind(wx.EVT_MENU, self.OnAbout, self.menu.about)

        #Splitter and panels
        splitter = wx.SplitterWindow(self)
        
        #Right panel
        rightP = wx.Panel(splitter)
        self.nb = wx.Notebook(rightP)
        self.tab_sample = sp.measurementPanel(self.nb, self.options)
        self.tab_gasbk = sp.measurementPanel(self.nb, self.options)
        self.tab_detbk = sp.measurementPanel(self.nb, self.options)
        self.tab_qc = sp.measurementPanel(self.nb, self.options)
        self.tab_calib = sp.measurementPanel(self.nb, self.options)
        self.tab_plot = wx.Panel(self.nb)
        self.nb.AddPage(self.tab_sample, "Sample")
        self.nb.AddPage(self.tab_gasbk, "Gasbk")
        self.nb.AddPage(self.tab_detbk, "Detbk")
        self.nb.AddPage(self.tab_qc, "QC")
        self.nb.AddPage(self.tab_calib, "Calib")
        self.nb.AddPage(self.tab_plot, "Plot")
        nb_sizer = wx.BoxSizer()
        nb_sizer.Add(self.nb, 1, wx.EXPAND)
        rightP.SetSizer(nb_sizer)
        
        #Left panel
        leftP = wx.Panel(splitter)
        self.nb2 = wx.Notebook(leftP)
        self.tab_log = wx.TextCtrl(self.nb2, style= wx.TE_MULTILINE)
        self.tab_log.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL, 0,
font)) 
        self.nb2.AddPage(self.tab_log, "Log")
        nb2_sizer = wx.BoxSizer()
        nb2_sizer.Add(self.nb2, 1, wx.EXPAND | wx.LEFT, border = 10)
        leftP.SetSizer(nb2_sizer)
        sys.stdout = self.tab_log
        
        #Split screen
        splitter.SplitVertically(leftP, rightP)
        splitter.SetMinimumPaneSize(100)
        split_sizer = wx.BoxSizer(wx.VERTICAL)
        split_sizer.Add(splitter, 1, wx.EXPAND)
        self.SetSizer(split_sizer)
        splitter.SetSashPosition(550)
        
    def OnQuit(self, event):
        """Close the main frame"""
        self.Close(True)
        
    def OnClearLog(self, event):
        """Clear the log window"""
        self.tab_log.Clear()
        
    def OnHelp(self,event):
        """Show the help menu in default browser"""
        fh, path = tempfile.mkstemp(suffix='.html')
        url = 'file://' + path
        with open(path, 'w') as fp:
            fp.write(h.HELPTEXT)
        webbrowser.open(url, new = 2)
        
    def OnAbout(self,event):
        """Show 'about' information"""
        info = format('This is openSpexF version %s\n\nAuthor: Anders Ringbom, Swedish Defence Research Agency (FOI)\n' %self.version)
        info += LICENSE
        ab.about(self, info).Show()
        
    def OnOptions(self, event):
        """Create the options panel with current options"""
        optpan = op.optionsPanel(self,"Options",self.options)
        optpan.Show()
        self.Bind(op.EVT_OPTIONS_SAVED, self.OnSaveOptions)
        
    def OnSaveOptions(self,event):
        """Immediate actions when options are changed"""
        #Set histogram plot options in measurement panels
        if self.tab_sample.lowerPanel.hglist:
            self.tab_sample.lowerPanel.hgbox.useCountrate = self.options.histScaleCountrate
        if self.tab_sample.lowerPanel.hblist:
            self.tab_sample.lowerPanel.hbbox.useCountrate = self.options.histScaleCountrate
        if self.tab_gasbk.lowerPanel.hglist:
            self.tab_gasbk.lowerPanel.hgbox.useCountrate = self.options.histScaleCountrate
        if self.tab_gasbk.lowerPanel.hblist:
            self.tab_gasbk.lowerPanel.hbbox.useCountrate = self.options.histScaleCountrate
        if self.tab_detbk.lowerPanel.hglist:
            self.tab_detbk.lowerPanel.hgbox.useCountrate = self.options.histScaleCountrate
        if self.tab_detbk.lowerPanel.hblist:
            self.tab_detbk.lowerPanel.hbbox.useCountrate = self.options.histScaleCountrate
        if self.tab_qc.lowerPanel.hglist:
            self.tab_qc.lowerPanel.hgbox.useCountrate = self.options.histScaleCountrate
        if self.tab_qc.lowerPanel.hblist:
            self.tab_qc.lowerPanel.hbbox.useCountrate = self.options.histScaleCountrate
        if self.tab_calib.lowerPanel.hglist:
            self.tab_calib.lowerPanel.hgbox.useCountrate = self.options.histScaleCountrate
        if self.tab_calib.lowerPanel.hblist:
            self.tab_calib.lowerPanel.hbbox.useCountrate = self.options.histScaleCountrate
        event.Skip()   
        
    def OnSaveMsetCSV(self,event):
        """Save a measuremetn set as .csv"""
        if self.mset == None:
            wx.MessageBox("No measurement set exists.","", wx.OK|wx.CENTRE)
            return
        di.SaveFile(self,self.mset.csv(),['CSV','csv'],['CSV files','CSV files'])
        
    def OnSaveObject(self, event):
        """Save a measurement or a measurement set to a pickle file"""
        if event.GetId()==menu.SAVE_MEAS:
            if self.bgsample == None:
                 wx.MessageBox("No measurement exists. Check data and analysis."
                                      ,"", wx.OK|wx.CENTRE)
                 return
            di.SavePickleFile(self,self.bgsample)
        if event.GetId()==menu.SAVE_DATASET:
            if self.mset == None:
                 wx.MessageBox("No measurement set exists.","", wx.OK|wx.CENTRE)
                 return
            di.SavePickleFile(self,self.mset)
                
    def OnReadObject(self, event):
        """Read a measurement or a measurement set from a pickle file"""
        if event.GetId()==menu.OPEN_MEAS:
           if self.bgsample != None:
               dlg = wx.MessageDialog(self, "Loaded measurement will be deleted. OK?"
                                      ,"", wx.YES_NO|wx.ICON_WARNING|wx.CENTRE)
               result = dlg.ShowModal()
               if result == wx.ID_NO:
                   return
           path, obj = di.ReadPickleFile(self)
           if path == None and obj == None:
               return
           if isinstance(obj, bgs.BGSample):
               self.bgsample = obj
               self.sample = self.bgsample.sample
               self.gasbk = self.bgsample.gasbk
               self.detbk = self.bgsample.detbk
               self.qc = self.bgsample.qc
               print("Loading new measurement")
               self.loadMeasurement()
           else:
               wx.MessageBox("Could not read a sample from selected file."
                                      ,"", wx.OK|wx.CENTRE)
               return
        if event.GetId()==menu.OPEN_DATASET:
           if self.mset != None:
               dlg = wx.MessageDialog(self, "Loaded measurement set will be deleted. OK?"
                                      ,"", wx.YES_NO|wx.ICON_WARNING|wx.CENTRE)
               result = dlg.ShowModal()
               if result == wx.ID_NO:
                   return
           path, obj = di.ReadPickleFile(self)
           if path == None and obj == None:
               return
           if isinstance(obj, xs.xenonSampleSet):
               self.mset = obj
               print("Loading new measurement set")
               self.OnPlotMset(0)
           else:
               wx.MessageBox("Could not read a measurement set from selected file."
                                      ,"", wx.OK|wx.CENTRE)
    
    def OnSavePHD(self,event):
         """Save loaded measurement as PHD-file"""
         if event.GetId()==menu.SAVE_SAMPLE_PHD and self.sample == None:
             wx.MessageBox("No sample exists.","", wx.OK|wx.CENTRE)
             return
         if event.GetId()==menu.SAVE_GASBK_PHD and self.gasbk == None:
             wx.MessageBox("No gas background exists.","", wx.OK|wx.CENTRE)
             return
         if event.GetId()==menu.SAVE_DETBK_PHD and self.detbk == None:
             wx.MessageBox("No detbk exists.","", wx.OK|wx.CENTRE)
             return
         if event.GetId()==menu.SAVE_QC_PHD and self.qc == None:
             wx.MessageBox("No QC exists.","", wx.OK|wx.CENTRE)
             return
         if event.GetId()==menu.SAVE_CALIB_PHD and self.calib == None:
             wx.MessageBox("No calib exists.","", wx.OK|wx.CENTRE)
             return
         if event.GetId()==menu.SAVE_SAMPLE_PHD:
             di.SaveFile(self,self.sample.IMSFormat(),['PHD','phd'],['IMS2.0 files','IMS 2.0 files'])
         elif event.GetId()==menu.SAVE_GASBK_PHD:
             di.SaveFile(self,self.gasbk.IMSFormat(),['PHD','phd'],['IMS2.0 files','IMS 2.0 files'])
         elif event.GetId()==menu.SAVE_DETBK_PHD:
             di.SaveFile(self,self.detbk.IMSFormat(),['PHD','phd'],['IMS2.0 files','IMS 2.0 files'])
         elif event.GetId()==menu.SAVE_QC_PHD:
             di.SaveFile(self,self.qc.IMSFormat(),['PHD','phd'],['IMS2.0 files','IMS 2.0 files'])
         elif event.GetId()==menu.SAVE_CALIB_PHD:
             di.SaveFile(self,self.calib.IMSFormat(),['PHD','phd'],['IMS2.0 files','IMS 2.0 files'])
         else:
             print("Wrong event id when saving PHD file.")
             
    def OnSaveReport(self,event):
        """Save measurement or measurement set report"""
        if event.GetId()==menu.SAVE_MEAS_REPORT:
            if self.bgsample == None:
                 wx.MessageBox("No measurement exists. Check data and analysis."
                                      ,"", wx.OK|wx.CENTRE)
                 return
            di.SaveFile(self,self.bgsample.bgmHandler.report(),['TXT','txt'],['TXT files','TXT files'])
        if event.GetId()==menu.SAVE_MEASSET_REPORT:
            if self.mset == None:
                 wx.MessageBox("No measurement set  exists."
                                      ,"", wx.OK|wx.CENTRE)
                 return
            di.SaveFile(self,self.mset.report,['TXT','txt'],['TXT files','TXT files'])
           
    def OnClearMea(self, event):
        """Delete measurement data and clear all plots"""
        print("Deleting measurement data")
        self.bgsample = None
        self.sample = None
        self.gasbk = None
        self.detbk = None
        self.qc = None
        self.calib = None
        self.tab_sample.clearData()
        self.tab_gasbk.clearData()
        self.tab_detbk.clearData()
        self.tab_qc.clearData()
        self.tab_calib.clearData()
        
        
    def OnUnmarkMset(self,event):
        """Unmark all measurements in the measurement set"""
        if self.mset == None:
             wx.MessageBox("Not measurement set loaded.","", wx.OK|wx.CENTRE)
             return
        for s in self.mset.set:
            s.marked = False
        if self.nb.FindPage(self.tab_plot) != wx.NOT_FOUND:
            self.mset.drawMarkers(self.tab_plot.figure)
        
    def OnClearMset(self, event):
        """Delete the measurement set and clear plots"""
        if self.mset == None:
             wx.MessageBox("Not measurement set loaded.","", wx.OK|wx.CENTRE)
             return
        dlg = wx.MessageDialog(self, "Loaded measurement set will be deleted. OK?"
                                      ,"", wx.YES_NO|wx.ICON_WARNING|wx.CENTRE)
        result = dlg.ShowModal()
        if result == wx.ID_NO:
            return
        print("Deleting measurement set")
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        self.tab_plot = wx.Panel(self.nb)
        self.nb.AddPage(self.tab_plot, "Plot")
        self.mset = None

    def OnCalcMset(self, event):
        """Calculate a measurement set from files"""
        if self.mset != None:
            dlg = wx.MessageDialog(self, "Loaded measurement set will be deleted. OK?"
                                      ,"", wx.YES_NO|wx.ICON_WARNING|wx.CENTRE)
            result = dlg.ShowModal()
            if result == wx.ID_NO:
                return
        txt = ''
        if self.options.writeResultFiles:
            txt += "Sample .pkl files will be written to:\n"
            txt += path.join(self.options.resultPath,'results')
            txt += '\n'
        if self.options.moveDataFiles:
            txt += 'Analyzed data files will be moved to:\n'
            txt += path.join(self.options.resultPath,'data')
        txt += "\n OK?"
        if self.options.moveDataFiles or self.options.writeResultFiles:
            dlg = wx.MessageDialog(self, txt,"", wx.YES_NO|wx.ICON_WARNING|wx.CENTRE)
            result = dlg.ShowModal()
            if result == wx.ID_NO:
                return
        self.mset = xs.xenonSampleSet(self.options.dataPath,
                                                       self.options.detbkPath,
                                                       resultPath = self.options.resultPath,
                                                       prog = True,
                                                       combine = self.options.combine, 
                                                       use_gasbk = self.options.analysisUseGB,
                                                       verbose = self.options.verbose,
                                                       useIngrowth=self.options.ingrowthCorr,
                                                       writeResults = self.options.writeResultFiles,
                                                       moveDataFiles = self.options.moveDataFiles)
        if self.mset.getNSamples() == 0:
            wx.MessageBox(format("Measurement set empty. Check data in %s" %self.options.dataPath),
                          "", wx.OK|wx.CENTRE)
            self.mset = None
            return 
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        self.tab_plot = pp.plotPanel(self.nb,
                                     self.mset.plotACTimeSeries(
                                        self.options.mSetPlot, 
                                        iso = self.options.mSetPlotIso, 
                                        hideMarked = self.options.hideMarked), 
                                        pickPoints = True,
                                        evtReciever = self)
        self.Bind(pp.EVT_POINT_PICKED, self.OnPointClicked)
        self.nb.AddPage(self.tab_plot, "Measurement Set")
        self.nb.ChangeSelection(5)
        
    def OnReportMset(self,evt):
        """Print a measurement set report."""
        if self.mset == None:
             wx.MessageBox("Not measurement set to report.","", wx.OK|wx.CENTRE)
             return
        self.mset.makeReport()
        print(self.mset.report)
        
    def OnPointClicked(self,evt):
         """Perform actions if a datapoint is clicked"""
         if not self.options.useMarkers:
             return
         index = evt.attr1
         figure = evt.attr2
         line_index = evt.attr3
         sp = self.mset.getPlottedSample(index,line_index)
         if sp != None:
             removed = sp.marked
             sp.marked = not sp.marked
             if not removed:
                 print(sp.makeInfo())
             if sp.resultPath != None and self.options.loadMarkedSample and not removed:
                 self.loadFileSample(sp.resultPath)
         self.mset.drawMarkers(figure)
        
    def OnPlotMset(self, event):
        """Plot a timeseries in the plot tab"""
        if self.mset == None:
             wx.MessageBox("Not measurement set to plot.","", wx.OK|wx.CENTRE)
             return
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        self.tab_plot = pp.plotPanel(self.nb,
                                     self.mset.plotACTimeSeries
                                     (self.options.mSetPlot, 
                                      iso = self.options.mSetPlotIso,
                                      hideMarked = self.options.hideMarked),
                                      pickPoints = True,
                                      evtReciever = self)
        self.nb.AddPage(self.tab_plot, "Measurement Set")
        self.nb.ChangeSelection(5)
        self.Bind(pp.EVT_POINT_PICKED, self.OnPointClicked)
    
    def OnFreqMset(self, event):
        """Plot frequency histograms in the plot tab"""
        if self.mset == None:
             wx.MessageBox("Not measurement set to plot.","", wx.OK|wx.CENTRE)
             return
        plotOpt = copy.copy(self.options.mSetPlot)
        if 'E' in plotOpt:
            plotOpt.remove('E')
        if 'L' in plotOpt:
            plotOpt.remove('L')
        if 'G' in plotOpt:
            plotOpt.remove('G')
        if len(plotOpt) > 1:
            wx.MessageBox("Please select only one plot option in the options menu.\n (AC, LC, MDC, A, LCA or MDA)"
                                      ,"", wx.OK|wx.CENTRE)
            return
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        self.tab_plot = pp.plotPanel(self.nb,
                                     self.mset.frequencyDistr
                                     (self.options.mSetPlot, 
                                      self.options.mSetPlotIso))
        self.nb.AddPage(self.tab_plot, "Frequency")
        self.nb.ChangeSelection(5)
    
    def OnAppendMset(self, event):
        """Append a measurement set to the currently loaded set"""
        if self.mset == None:
             wx.MessageBox("Not measurement set to append to."
                                      ,"", wx.OK|wx.CENTRE)
             return
        path, mset = di.ReadPickleFile(self)
        if mset != None:
            print("Appending measurement set")
            self.mset.appendSet(mset)
            self.OnPlotMset(0)
    
    def OnPlotParam(self,event):
        if self.mset == None:
            wx.MessageBox("Not measurement set to plot.","", wx.OK|wx.CENTRE)
            return
        opt = []
        if self.menu.plotXeVol.IsChecked():
            opt.append('xeVol')
        if self.menu.plotXeYield.IsChecked():
            opt.append('xeYield')
        if self.menu.plotAirVol.IsChecked():
            opt.append('airVol')
        if self.menu.plotErrors.IsChecked():
            showErr = True
        else:
            showErr = False
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        self.tab_plot = pp.plotPanel(self.nb,self.mset.plotQuantity(opt, showErr))
        self.nb.AddPage(self.tab_plot, 'Parameters')
        self.nb.ChangeSelection(5)
        
    def OnMircMset(self,event):
        """Plot a MIRC plot in the plot tab"""
        if self.mset == None:
            wx.MessageBox("Not measurement set to plot.","", wx.OK|wx.CENTRE)
            return
        page =  self.nb.FindPage(self.tab_plot)
        self.nb.DeletePage(page)
        if event.GetId()==menu.MSET_MIRC3: opt = 3
        if event.GetId()==menu.MSET_MIRC4: opt = 4
        self.tab_plot = pp.plotPanel(self.nb,self.mset.MIRC(opt), 
                                     pickPoints = True, evtReciever = self)
        self.nb.AddPage(self.tab_plot, "MIRC" + str(opt))
        self.nb.ChangeSelection(5)
        self.Bind(pp.EVT_POINT_PICKED, self.OnPointClicked)
            
    def OnAnalyze(self, event):
        """
        Create a BGSample if enough data loaded. 
        Do a BGM- analysis if no input errors found.
        """
        self.bgsample = None
        bgmOK = False
        tweakParam = [0,1,0,1]
        if self.sample != None and self.detbk != None:
            tweak = False
            if self.options.tweakParameters[0]:
                tweakParam[0] = self.options.tweakParameters[1]
                tweak = True
            if self.options.tweakParameters[2]:
                tweakParam[1] = self.options.tweakParameters[3]
                tweak = True
            if self.options.tweakParameters[4]:
                tweakParam[2] = self.options.tweakParameters[5]
                tweak = True
            if self.options.tweakParameters[6]:
                tweakParam[3] = self.options.tweakParameters[7]
                tweak = True
            self.sample.resetROIs()
            if tweak:
                print("Tweaking sample using [beta offset, beta tweak, gamma offset, gamma tweak] = ", tweakParam)
                self.tweakedLastSample = True
                self.sample.calibrate(tweakParam)
            else:
                if self.tweakedLastSample:
                    print("Recalibrating sample since it was tweaked the last time")
                    self.sample.calibrate()
                    self.tweakedLastSample = False
            if not self.sample.errors and not self.detbk.errors:
                if self.options.analysisUseGB and self.gasbk != None:
                    if not self.gasbk.errors:
                        self.gasbk.resetROIs()
                        if tweak:
                            self.gasbk.calibrate(tweakParam)
                            self.tweakedLastGasbk = True
                        else:
                            if self.tweakedLastGasbk:
                                self.gasbk.calibrate()
                                self.tweakedLastGasbk = False
                        self.bgsample = bgs.BGSample(self.sample, self.detbk, self.gasbk, self.qc)
                        print("Created measurement with gas background.")
                        bgmOK = not ((self.bgsample.sample.status & bg.ZERO_GROSS) or\
                            (self.bgsample.gasbk.status & bg.ZERO_GROSS))
                if not self.options.analysisUseGB:
                    self.bgsample = bgs.BGSample(self.sample, self.detbk, qc = self.qc)
                    print("Created measurement without gas background.")
                    bgmOK = not (self.bgsample.sample.status & bg.ZERO_GROSS)
        if bgmOK:
                self.bgsample.bgmAnalysis(self.options.combine, self.options.verbose, self.options.ingrowthCorr)
                print(self.bgsample.bgmHandler.report())
                self.tab_sample.loadData(self.sample, self.bgsample, useCountrate = self.options.histScaleCountrate)
                self.tab_gasbk.loadData(self.gasbk, useCountrate = self.options.histScaleCountrate)
                self.nb.ChangeSelection(0)
        else:
            wx.MessageBox("Data error. Check data and analysis options."
                                      ,"", wx.OK|wx.CENTRE)         
            
    def OpenPHD(self,event):
        """Open a PHD-file and load data into a measurementPanel"""
        pathname, data = di.ReadFile(self,['PHD','phd'],['IMS2.0 files','IMS2.0 files'])
        if data is None:
            return
        else:
            if event.GetId()==menu.OPEN_SAMPLE:
                if (ut.getDataType(pathname) == 'SAMPLEPHD' and not
                    self.options.ignoreFiletype) or self.options.ignoreFiletype:
                    self.sample = bg.BGMeasurement(self.options.fileStationType, pathname)
                    self.tab_sample.loadData(self.sample, useCountrate = self.options.histScaleCountrate)
                    self.nb.ChangeSelection(0)
                else:
                     wx.MessageBox("File is not SAMPLEPHD. This can be overrun by selecting the \"Ignore PHD filetype\" option."
                                      ,"", wx.OK|wx.CENTRE) 
            if event.GetId()==menu.OPEN_GASBK:
                if (ut.getDataType(pathname) == 'GASBKPHD' and not
                    self.options.ignoreFiletype) or self.options.ignoreFiletype:
                        self.gasbk = bg.BGMeasurement(self.options.fileStationType, pathname)
                        self.tab_gasbk.loadData(self.gasbk, useCountrate = self.options.histScaleCountrate)
                        self.nb.ChangeSelection(1)
                else:
                      wx.MessageBox("File is not GASBKPHD. This can be overrun by selecting the \"Ignore PHD filetype\" option."
                                      ,"", wx.OK|wx.CENTRE) 
            if event.GetId()==menu.OPEN_DETBK:
                if (ut.getDataType(pathname) == 'DETBKPHD' and not
                    self.options.ignoreFiletype) or self.options.ignoreFiletype:
                        self.detbk = bg.BGMeasurement(self.options.fileStationType, pathname)
                        self.tab_detbk.loadData(self.detbk, useCountrate = self.options.histScaleCountrate)
                        self.nb.ChangeSelection(2)
                else:
                    print("Wrong filetype")
            if event.GetId()==menu.OPEN_QC:
                if (ut.getDataType(pathname) == 'QCPHD' and not
                    self.options.ignoreFiletype) or self.options.ignoreFiletype:
                        self.qc = bg.BGMeasurement(self.options.fileStationType, pathname)
                        self.tab_qc.loadData(self.qc, useCountrate = self.options.histScaleCountrate)
                        self.nb.ChangeSelection(3)
                else:
                     wx.MessageBox("File is not QCPHD. This can be overrun by selecting the \"Ignore PHD filetype\" option."
                                      ,"", wx.OK|wx.CENTRE) 
            if event.GetId()==menu.OPEN_CALIB:
                if (ut.getDataType(pathname) == 'CALIBPHD' and not
                    self.options.ignoreFiletype) or self.options.ignoreFiletype:
                        self.calib = bg.BGMeasurement(self.options.fileStationType, pathname)
                        self.tab_calib.loadData(self.calib, useCountrate = self.options.histScaleCountrate)
                        self.nb.ChangeSelection(4)
                else:
                     wx.MessageBox("File is not CALILBPHD. This can be overrun by selecting the \"Ignore PHD filetype\" option."
                                      ,"", wx.OK|wx.CENTRE) 
                    
    def PrintPHD(self,event):
        """Print loaded data on IMS -format"""
        if event.GetId()==menu.PRINT_SAMPLE:
            if self.sample == None:
                wx.MessageBox("No Sample loaded","", wx.OK|wx.CENTRE)
            else:
                print(self.sample.IMSFormat())
        if event.GetId()==menu.PRINT_GASBK:
            if self.gasbk == None:
                wx.MessageBox("No Gasbk loaded","", wx.OK|wx.CENTRE)
            else:
                print(self.gasbk.IMSFormat())
        if event.GetId()==menu.PRINT_DETBK:
            if self.detbk == None:
                wx.MessageBox("No Detbk loaded","", wx.OK|wx.CENTRE)
            else:
                print(self.detbk.IMSFormat())
        if event.GetId()==menu.PRINT_QC:
            if self.qc == None:
                wx.MessageBox("No QC loaded","", wx.OK|wx.CENTRE)
            else:
                print(self.qc.IMSFormat())
        if event.GetId()==menu.PRINT_CALIB:
            if self.calib == None:
                wx.MessageBox("No Calib loaded","", wx.OK|wx.CENTRE)
            else:
                print(self.calib.IMSFormat())     
                    
    def loadMeasurement(self):
        """Load measurement data into the various tabs"""
        if self.bgsample != None:
            self.tab_sample.loadData(self.bgsample.sample, self.bgsample, useCountrate = self.options.histScaleCountrate)
            self.tab_gasbk.loadData(self.bgsample.gasbk, useCountrate = self.options.histScaleCountrate)
            self.tab_detbk.loadData(self.bgsample.detbk, useCountrate = self.options.histScaleCountrate)
            self.tab_qc.loadData(self.bgsample.qc, useCountrate = self.options.histScaleCountrate)
            self.nb.ChangeSelection(0)
    
    def loadFileSample(self,filePath):
       wait =  wx.BusyCursor()
       try:
           with open(filePath, 'rb') as file:
               print("Reading file %s" % filePath)
               obj = pickle.load(file)
       except IOError:
            print("Cannot open file %s\n" %filePath)
            obj = None
            del wait
            return
       self.bgsample = obj
       self.sample = self.bgsample.sample
       self.gasbk = self.bgsample.gasbk
       self.detbk = self.bgsample.detbk
       self.qc = self.bgsample.qc
       self.loadMeasurement()
       self.nb.ChangeSelection(0)
       del wait
                    
    def clearMeasurementPanels(self):
        """Clear all measurement tabs"""
        self.tab_sample.clearData()
        self.tab_gasbk.clearData()
        self.tab_detbk.clearData()
        self.tab_qc.clearData()
        self.tab_calib.clearData()


app = wx.App(False)
mainFrame(None,"openSpex " + VERSION + " BGM " + bgm.VERSION).Show()
app.MainLoop()
del app


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sun Mar 29 12:10:43 2020

@author: ringbom

This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.gridspec import GridSpec
import copy

from open_spex import utils as ut

# (1 << k) | n will set bit k in n
MSG_TYPE_ERR = (1 << 0) | 0
MSG_ID_ERR = (1 << 1) | 0
DATA_TYPE_ERR = (1 << 2) | 0 
HEADER_ERR = (1 << 3) | 0
COMMENT_ERR = (1 << 4) | 0
COLL_ERR = (1 << 5) | 0
ACQ_ERR = (1 << 6) | 0
PROC_ERR = (1 << 7) | 0
CAL_ERR = (1 << 8) | 0
G_ENERGY_ERR = (1 << 9) | 0 
B_ENERGY_ERR = (1 << 10) | 0
G_RES_ERR = (1 << 11) | 0
B_RES_ERR = (1 << 12) | 0
G_EFF_ERR = (1 << 13) | 0
ROI_LIM_ERR = (1 << 14) | 0
BG_EFF_ERR = (1 << 15) | 0
RATIO_ERR = (1 << 16) | 0
G_SPECT_ERR = (1 << 17) | 0
B_SPECT_ERR = (1 << 18) | 0
BG_SPECT_ERR = (1 << 19) | 0
ZERO_GROSS = (1 << 21) | 0
ROI_CHAN_LIM_ERR = (1 << 22) | 0
CERTIFICATE_ERR = (1 << 23) | 0

class BGMeasurement(object):
    """ 
    Class handling a beta-gamma measurement.
    Parameters
    ==========
    stationType      'CUBE' or 'SAUNA'
    fileName         filename if data source is a IMS2.0 text file
    verbose          More printout if 'True'. Default is false.
    makeHist         If 'True' (default), make histograms. If 'False', no histograms are made.
                     This can be useful when analysis is done without plotting spectra 
                     (saves time and memory).
    calibrate        'True' (default): calibrate the sample. 
         
    The following histograms are created:
        
    beta_gamma_hist
    gamma_hist
    beta_hist
    beta_coinc_hist 
    gamma_coinc_hist 
    beta350_hist 
    beta250_hist
    beta81_hist 
    beta30_hist 
    
    """
    def __init__(self, stationType, fileName, verbose = False, makeHist = True, calibrate = True):
     
        self.verbose = verbose
        self.status = 0
        self.stationType = stationType
        self.makeHist = makeHist
        self.sampleId = None
        self.fileName = fileName
        if self.verbose:
            print("Parsing PHD-file ",self.fileName)
        with open(self.fileName, 'r') as f:
            data = f.read()
            self.parsePHDData(data)
        self.dataSource = 'FILE'
        if self.msg_type == 'CUBE':
            self.stationType = 'CUBE'
        self.checkStatus()
        if self.errors:
            return
        #self.tweaked = False
        if calibrate:
            self.calibrate()
            #store calibration data to allow changes later
            self.calib_date_orig = copy.deepcopy(self.calib_date)
            self.gEnergy_orig = copy.deepcopy(self.gEnergy)
            self.gResolution_orig = copy.deepcopy(self.gResolution)
            self.bEnergy_orig = copy.deepcopy(self.bEnergy)
            self.bResolution_orig = copy.deepcopy(self.bResolution)
            self.ROI_limits_ene_orig = copy.deepcopy(self.ROI_limits_ene)
            self.ROI_limits_chan_orig = copy.deepcopy(self.ROI_limits_chan)
            self.bg_eff_orig = copy.deepcopy(self.bg_eff)
            self.ratio_orig = copy.deepcopy(self.ratio)
            #self.gEfficiency_orig = self.gEfficiency
        if self.makeHist:
            self.makeInfo()
        self.db = None     
        self.buildIMSFileName()
    
            
    def buildIMSFileName(self):
        """Constructs the file name used to save data in IMS-format"""
        dt = self.data_type.replace('PHD','')
        if hasattr(self,"collstart_time"):
            self.phdFileName = format('%s-%s-%d-%s.PHD'%(self.detector, 
                                  self.collstart_time.strftime("%Y-%m-%d--%H-%M-%S"),
                                  self.acq_realtime, dt))
        else:
            self.phdFileName = format('%s-%s-%d-%s.PHD'%(self.detector, 
                                  self.acq_start.strftime("%Y-%m-%d--%H-%M-%S"),
                                  self.acq_realtime, dt))
        
    def parsePHDData(self,data):
        """
        Parse data in IMS2.0 - format
        
        Parameters
        ----------
        data          text string in IMS2.0 format
        """
        nok1 = 1
        nok2 = 1
        nok3 = 1
        for line in data.splitlines():
            w = line.split()
            if 'MSG_TYPE' in line:
                self.msg_type = w[1]
                nok1 = 0
            if 'MSG_ID' in line:
                self.msg_id = line.replace("MSG_ID ","")
                nok2 = 0
            if 'DATA_TYPE' in line:
                self.data_type = w[1]
                nok3 = 0
        self.status += nok1*MSG_TYPE_ERR + nok2*MSG_ID_ERR + nok3*DATA_TYPE_ERR
        #
        # Header block
        #
        block = ut.get_block(data,'#Header')
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    self.station = w[0]
                    self.detector = w[1]
                    self.qual = w[4]
                if l == 2:
                    self.srid = w[0]
                if l == 3:
                    self.sample_id = w[0]
                    self.detbk_id = w[1]
                    self.gasbk_id = w[2]
                if l == 4:
                    date = w[0] + " " + w[1]
                    d = ut.readDate(date)
                    if not d:
                        break
                    else:
                        self.transmit_time = d
                l += 1
        else:
            self.status += HEADER_ERR
        #
        # Comment block
        #
        block = ut.get_block(data,'#Comment')
        self.comment = ''
        if len(block) > 0:
            i = 0
            for line in block:
                if i > 0:
                    self.comment += line
                i += 1
        else:
            self.status += COMMENT_ERR
        #
        # Collection block
        #
        if self.data_type == 'SAMPLEPHD' or self.data_type == 'SPIKEPHD' or self.data_type == 'REGENPHD':
            block = ut.get_block(data,'#Collection')
            if len(block) > 0:
                l = 0
                for line in block:
                    w = line.split()
                    if l == 1:
                        date = w[0] + " " +  w[1]
                        d = ut.readDate(date)
                        if not d:
                            break
                        else:
                            self.collstart_time = d
                        date = w[2] + " " + w[3]
                        d = ut.readDate(date)
                        if not d:
                            break
                        else:
                            self.collstop_time = d
                        self.air_volume = float(w[4])
                    l += 1
            else:
                self.status += COLL_ERR
        #
        # Acquisition block
        #
        block = ut.get_block(data,'#Acquisition')
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    date = w[0] + " " +  w[1]
                    d = ut.readDate(date)
                    if not d:
                        break
                    else:
                        self.acq_start = d
                    self.acq_realtime = float(w[2])
                    self.acq_livetime = float(w[3])
                l += 1
        else:
            self.status += ACQ_ERR
        #
        # Processing block
        #
        if self.data_type == 'SAMPLEPHD' or self.data_type == 'SPIKEPHD'or self.data_type == 'REGENPHD':
            block = ut.get_block(data,'#Processing')
            if len(block) > 0:
                l = 0
                for line in block:
                    w = line.split()
                    if l == 1:
                        self.xe_vol = float(w[0])
                        self.xe_vol_err = float(w[1])
                    if l == 2:
                        self.xe_yield = float(w[0])
                        self.xe_yield_err = float(w[1])
                    if l == 3:
                        self.archive_nbr = w[0]
                    l += 1
            else:
                self.status += PROC_ERR
        #
        # Calibration block
        #
        block = ut.get_block(data,'#Calibration')
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    date = w[0] + " "  + w[1]
                    d = ut.readDate(date)
                    if not d:
                        break
                    else:
                        self.calib_date = d
                l += 1
        else:
            self.status += CAL_ERR
        #
        # g_Energy block
        #
        block = ut.get_block(data,'#g_Energy')
        self.gEnergy = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.gEnergy.append([float(w[0]), float(w[1]), float(w[2])])
                l += 1
        else:
            self.status += G_ENERGY_ERR

        #
        # g_Resolution block
        #
        block = ut.get_block(data,'#g_Resolution')
        self.gResolution = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.gResolution.append([float(w[0]), float(w[1]), float(w[2])])
                l += 1
        else:
            self.status += G_RES_ERR
        #
        # b_Energy block
        #
        block = ut.get_block(data,'#b_Energy')
        self.bEnergy = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.bEnergy.append([float(w[0]), float(w[2]), float(w[3])])
                l += 1
        else:
            self.status += B_ENERGY_ERR
        #
        # b_Resolution block
        #
        block = ut.get_block(data,'#b_Resolution')
        self.bResolution = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.bResolution.append([float(w[0]), float(w[1]), float(w[2])])
                l += 1
        else:
            self.status += B_RES_ERR
        #
        # g_Efficiency block
        #
        block = ut.get_block(data,'#g_Efficiency')
        self.gEfficiency = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.gEfficiency.append([float(w[0]), float(w[1]), float(w[2])])
                l += 1
        else:
            self.status += G_EFF_ERR
        #
        # ROI_Limits block
        #
        block = ut.get_block(data,'#ROI_Limits')
        self.ROI_limits_ene = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.ROI_limits_ene.append([int(w[0]), float(w[1]), \
                                    float(w[2]), float(w[3]), float(w[4])])
                l += 1
        else:
            self.status += ROI_LIM_ERR
        #
        # ROI_Chan_Limits block (for CUBE only)
        #
        self.ROI_limits_chan = []
        if self.msg_type == 'CUBE':
            block = ut.get_block(data,'#ROI_Chan_Limits')
            #self.ROI_limits_chan = []
            if len(block) > 0:
                l = 0
                for line in block:
                    w = line.split()
                    if l > 0:
                        self.ROI_limits_chan.append([int(float(w[0])), int(float(w[1])), \
                                        int(float(w[2])), int(float(w[3])), int(float(w[4]))])
                    l += 1
            else:
                self.status += ROI_CHAN_LIM_ERR
        #
        # b-gEfficiency block
        #
        block = ut.get_block(data,'#b-gEfficiency')
        self.bg_eff = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.bg_eff.append([w[0], int(w[1]), float(w[2]), float(w[3])])
                l += 1
            if len(self.bg_eff) != 9:
                 self.status += BG_EFF_ERR
        else:
            self.status += BG_EFF_ERR
        #
        # Ratios block
        #
        block = ut.get_block(data,'#Ratios')
        self.ratio = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l > 0:
                    self.ratio.append([w[0], int(w[1]), int(w[2]), float(w[3]), float(w[4])])
                l += 1
            if (len(self.ratio) != 16 and len(self.ratio) != 18):
                self.status += RATIO_ERR
        else:
            self.status += RATIO_ERR
        #
        # g_Spectrum block
        #
        block = ut.get_block(data,'#g_Spectrum')
        self.gamma = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    self.ngamma_chan = int(w[0])
                    self.gamma_maxenergy = float(w[1])
                if l > 1:
                    i = 0
                    for word in w:
                        if i > 0:
                            self.gamma.append(int(word))
                        i += 1
                l += 1
        else:
            self.status += G_SPECT_ERR
        #
        # b_Spectrum block
        #
        block = ut.get_block(data,'#b_Spectrum')
        self.beta = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    self.nbeta_chan = int(w[0])
                    self.beta_maxenergy = float(w[1])
                if l > 1:
                    i = 0
                    for word in w:
                        if i > 0:
                            self.beta.append(int(word))
                        i += 1
                l += 1
        else:
            self.status += B_SPECT_ERR
        #
        # Histogram block
        #
        block = ut.get_block(data,'#Histogram')
        self.beta_gamma = []
        if len(block) > 0:
            l = 0
            for line in block:
                w = line.split()
                if l == 1:
                    self.nbg_gamma_chan = int(w[0])
                    self.nbg_beta_chan = int(w[1])
                    self.nbg_gamma_maxene = int(w[2])
                    self.nbg_beta_maxene = int(w[3])
                if l > 1:
                    for word in w:
                        self.beta_gamma.append(int(word))
                l += 1
        else:
            self.status += BG_SPECT_ERR
        #
        # Certification block
        #
        block = ut.get_block(data,'#Certificate')
        self.certificate = ''
        if len(block) > 0:
             i = 0
             for line in block:
                 if i > 0: 
                     self.certificate += line
                 i += 1
        else:
            self.status += CERTIFICATE_ERR

    def checkStatus(self):
        """Check status of data parsing"""
        self.warnings = False
        self.errors = False
        if self.status & MSG_TYPE_ERR:
            print("Error in MSG_TYPE")
            self.errors = True
        if self.status & MSG_ID_ERR:
            print("Error in MSG_ID")
            self.errors = True
        if self.status & DATA_TYPE_ERR:
            print("Error in  DATA_TYPE")
            self.errors = True
        if self.status & HEADER_ERR:
            print("Error in  #Header")
            self.errors = True
        if self.status & COMMENT_ERR:
            print("Warning: #Comment not found")
            self.warnings = True
        if self.status & COLL_ERR:
            print("Error in #Collection")
            self.errors = True
        if self.status & ACQ_ERR:
            print("Error in  #Acquisition")
            self.errors = True
        if self.status & PROC_ERR:
            print("Error in  #Processing")
            self.errors = True
        if self.status & CAL_ERR:
            print("Warning: #Calibration not found")
            self.warnings = True
        if self.status & G_ENERGY_ERR:
            print("Warning: #g_Energy not found")
            self.warnings = True
        if self.status & G_RES_ERR:
            print("Warning: #g_Resolution not found")
            self.warnings = True
        if self.status & B_ENERGY_ERR:
            print("Warning: #b_Energy not found in")
            self.warnings = True
        if self.status & B_RES_ERR:
            print("Warning: #b_Resolution not found")
            self.warnings = True
        if self.status & G_EFF_ERR:
            print("Warning: #g_Efficiency not found")
            self.warnings = True
        if self.status & ROI_LIM_ERR:
            print("Warning: #ROI_Limits not found")
            self.warnings = True
        if self.status & ROI_CHAN_LIM_ERR:
            print("Warning: #ROI_Chan_Limits not found")
            self.warnings = True
        if self.status & BG_EFF_ERR:
            print("Warning: #b-gEfficiency incomplete")
            self.warnings = True
        if self.status & RATIO_ERR:
            print("Warning: #Ratios incomplete")
            self.warnings = True
        if self.status & G_SPECT_ERR:
            print("Error in  #g_Spectrum")
            self.errors = True
        if self.status & B_SPECT_ERR:
            print("Error in #b_Spectrum")
            self.errors = True
        if self.status & BG_SPECT_ERR:
            print("Error in #Histogram")
            self.errors = True
        if self.status & CERTIFICATE_ERR:
            #print("Warning: no #Certificate block")
            self.warnings = True
        if self.status & ZERO_GROSS:
            self.errors = True
        # Other errors:
        if self.errors == False:
            if self.acq_realtime == 0 or self.acq_livetime == 0:
                print("Error: acq_livetime =  %f acq_realtime = %f " 
                      %(self.acq_livetime, self.acq_realtime))
                self.status += ACQ_ERR
                self.errors = True
            if sum(self.beta_gamma) == 0:
                print("Error: Beta-gamma histogram is empty.")
                self.status += BG_SPECT_ERR
                self.errors = True
            if self.data_type == 'SAMPLEPHD' or \
                                    self.data_type == 'REGENPHD' or \
                                    self.data_type == 'SPIKEPHD':
                if self.xe_vol <= 0:
                    print("Warning: Xenon volume %6.6f <= 0 " %self.xe_vol)
                    self.status += PROC_ERR
                    self.warnings = True
                if self.xe_vol > 0. and self.xe_vol <= 0.3:
                    print("Warning: Xenon volume= %6.6f" %self.xe_vol)
                    self.warnings = True
                coll_time =  (self.collstop_time - 
                              self.collstart_time).total_seconds()
                if coll_time < 0 or coll_time > 24*3600:
                    print("Error: Collection time = %d" %coll_time)
                    self.status += COLL_ERR
                    self.errors = True
    
      
    def printLoadedCalibration(self):
        """Print calibration data."""
        txt = format("\n\nCalibration for a %s %s measurement with acquisition start %s\n"
                     %(self.stationType, self.data_type, self.acq_start))
        txt += "===========================================================================================\n"
        txt += format("Calibration date: %s\n" %self.calib_date)
        txt += format("\nGamma energy\n")
        for d in self.gEnergy:
            txt += format("%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2]))
        txt += format("\nGamma resolution\n")
        for d in self.gResolution:
            txt += format("%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2]))
        txt += format("\nElectron energy\n")
        for d in self.bEnergy:
            txt += format("%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2]))
        txt += format("\nElectron resolution\n")
        for d in self.bResolution:
            txt += format("%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2]))
        txt += format("\nROI limits in keV\n")
        for d in self.ROI_limits_ene:
            txt += format("%d\t%3.3f\t%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2], d[3], d[4]))
        if self.stationType == 'CUBE':
            txt += format("\nROI limits in channels\n")
            for d in self.ROI_limits_chan:
                txt += format("%d\t%3.3f\t%3.3f\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2], d[3], d[4]))
        txt += format("\nROI efficiency\n")
        for d in self.bg_eff:
            txt += format("%s\t%d\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2], d[3]))
        txt += format("\nInterference ratios\n")
        for d in self.ratio:
            txt += format("%s\t%d\t%d\t%3.3f\t%3.3f\n" %(d[0], d[1], d[2], d[3], d[4]))
        print(txt)
        
                        
    def calibrate(self, tweakParam = None):
        """
        Calibrates beta- and gamma energy scales. 
        Calculates ROI limits in channels and ROI counts. 
        Makes histograms and calculates countrates
        
        Parameters
        ==========
        tweakParam = [offsetBeta, tweakBeta,offsetGamma,tweakGamma]; default = None
        """
        #Calibrate
        self.calMethod = "Station"
        self.calibrated = False
        if len(self.gEnergy) < 2 or len(self.bEnergy) < 2:
            print("Warning: Calibration data is missing. Cannot calibrate.")
            self.ROI_limits_chan = []
            self.roi_counts = []
            self.calMethod = "Not performed"
            return
        self.pol_g_ene_chan = ut.fit_polynom(self.gEnergy,2)
        self.pol_b_ene_chan = ut.fit_polynom(self.bEnergy,2)
        self.pol_g_ene_fwhm = ut.fit_polynom(self.gResolution,2)
        self.pol_b_ene_fwhm = ut.fit_polynom(self.bResolution,2)
        #if self.stationType == 'CUBE' and self.dataSource != 'FILE':
        #This requires that ROIs are given in at least one of energies and channels:
        if len(self.ROI_limits_ene) == 0:
            self.ROI_limits_ene = []
            for x in self.ROI_limits_chan:
                self.ROI_limits_ene.append([x[0],ut.invert_pol(x[1],self.pol_b_ene_chan), \
                  ut.invert_pol(x[2],self.pol_b_ene_chan), ut.invert_pol(x[3],self.pol_g_ene_chan), \
                  ut.invert_pol(x[4],self.pol_g_ene_chan)])
        if len(self.ROI_limits_chan) == 0:
        #if self.stationType == 'SAUNA' or self.dataSource == 'FILE':
            #Calculate ROI limits in int channels (rounded)
            self.ROI_limits_chan = []
            for x in self.ROI_limits_ene:
                self.ROI_limits_chan.append([x[0],int(self.pol_b_ene_chan(x[1])+0.5), \
                  int(self.pol_b_ene_chan(x[2])+0.5), int(self.pol_g_ene_chan(x[3])+0.5), \
                  int(self.pol_g_ene_chan(x[4])+0.5)])
            self.ROI_limits_chan_orig = copy.deepcopy(self.ROI_limits_chan)
        #Tweak ROIs if tweakParam exists
        if tweakParam != None:
            self.tweaked = True
            self.calMethod = "Tweaked"
            for roi in self.ROI_limits_chan:
                roi[1] = roi[1]*tweakParam[1] + tweakParam[0]
                roi[2] = roi[2]*tweakParam[1] + tweakParam[0]
                roi[3] = roi[3]*tweakParam[3] + tweakParam[2]
                roi[4] = roi[4]*tweakParam[3] + tweakParam[2]
        else:
            self.tweaked = False
        self.tweakParam = tweakParam
        #Calculate ROI counts
        self.roi_counts = np.zeros(len(self.ROI_limits_chan))
        for g in range(self.nbg_gamma_chan):
            for b in range(self.nbg_beta_chan):
                cont = self.beta_gamma[g*self.nbg_beta_chan + b]
                i = 0
                for roi in self.ROI_limits_chan:
                    if b >= roi[1] and b< roi[2] and g >= roi[3] and g < roi[4]: 
                        self.roi_counts[i] += cont
                    i += 1
        if sum(self.roi_counts) == 0:
            print("Error: Zero gross counts")
            self.status += ZERO_GROSS
        #Make histograms
        if self.makeHist:
            self.beta_gamma_hist = ut.histogram2D([self.nbg_gamma_chan, self.nbg_beta_chan], 
                                                  self.beta_gamma, 
                                                  title = 'Beta-Gamma', 
                                                  xlabel = 'Electron energy (chan)', 
                                                  ylabel = 'Gamma energy (chan)')
            self.gamma_hist = ut.histogram1D(self.ngamma_chan,self.gamma,
                                self.data_type + ' ' + "Gamma singles", 'Energy (chan)', 'Counts/bin',
                                liveTime = self.acq_livetime)
            self.gammaSinglesCR = sum(self.gamma_hist.data[0:])
            self.beta_hist = ut.histogram1D(self.nbeta_chan,self.beta,
                                self.data_type + ' ' + "Beta singles", 'Energy (chan)', 'Counts/bin',
                                liveTime = self.acq_livetime)
            self.betaSinglesCR = sum(self.gamma_hist.data[0:])
            self.coincCR = sum(self.beta_gamma[0:])
            beta_350 = np.zeros(self.nbg_beta_chan)
            beta_250 = np.zeros(self.nbg_beta_chan)
            beta_81 = np.zeros(self.nbg_beta_chan)
            beta_30 = np.zeros(self.nbg_beta_chan)
            gamma_coinc = np.zeros(self.nbg_gamma_chan)
            beta_coinc = np.zeros(self.nbg_beta_chan)
            for g in range(self.nbg_gamma_chan):
                for b in range(self.nbg_beta_chan):
                        beta_coinc[b] += self.beta_gamma[g*self.nbg_beta_chan + b]
                if g >= self.ROI_limits_chan[0][3] and g < self.ROI_limits_chan[0][4]: 
                    for b in range(self.nbg_beta_chan):
                        beta_350[b] += self.beta_gamma[g*self.nbg_beta_chan + b]
                if g >= self.ROI_limits_chan[1][3] and g < self.ROI_limits_chan[1][4]: 
                    for b in range(self.nbg_beta_chan):
                        beta_250[b] += self.beta_gamma[g*self.nbg_beta_chan + b]
                if g >= self.ROI_limits_chan[2][3] and g < self.ROI_limits_chan[2][4]: 
                    for b in range(self.nbg_beta_chan):
                        beta_81[b] += self.beta_gamma[g*self.nbg_beta_chan + b]
                if g >= self.ROI_limits_chan[3][3] and g < self.ROI_limits_chan[3][4]: 
                    for b in range(self.nbg_beta_chan):
                        beta_30[b] += self.beta_gamma[g*self.nbg_beta_chan + b]
            for b in range(self.nbg_beta_chan):
                for g in range(self.nbg_gamma_chan):
                    #gamma_coinc[g] += self.beta_gamma[g*self.nbg_beta_chan + b]
                    # Note change back!!!
                    if b > 0 and g > 0: 
                            gamma_coinc[g] += self.beta_gamma[g*self.nbg_beta_chan + b]
            self.beta350_hist = ut.histogram1D(self.nbg_beta_chan,beta_350,
                                        self.data_type + ' ' + "Beta 350 keV gamma", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
            self.beta250_hist = ut.histogram1D(self.nbg_beta_chan,beta_250,
                                        self.data_type + ' ' + "Beta 250 keV gamma", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
            self.beta81_hist = ut.histogram1D(self.nbg_beta_chan,beta_81,
                                        self.data_type + ' ' + "Beta 81 keV gamma", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
            self.beta30_hist = ut.histogram1D(self.nbg_beta_chan,beta_30,
                                        self.data_type + ' ' + "Beta 30 keV gamma", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
            self.beta_coinc_hist = ut.histogram1D(self.nbg_beta_chan,beta_coinc,
                                        self.data_type + ' ' + "Beta total coincidence", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
            self.gamma_coinc_hist = ut.histogram1D(self.nbg_gamma_chan,gamma_coinc,
                                        self.data_type + ' ' + "Gamma total coincidence", 'Energy (chan)', 'Counts/bin',
                                        liveTime = self.acq_livetime)
        self.calibrated = True
        self.makeInfo()
        
    def resetROIs(self):
        """Set ROI limits to original calibration"""
        self.ROI_limits_chan = copy.deepcopy(self.ROI_limits_chan_orig)

    def get1DHistList(self, opt):
        """
        Make a list with the histograms. 
        This can be used in plotting menus etc. 
        
        Parameters
        ----------
        opt          'beta' or 'gamma'
        """
        if opt != 'gamma' and opt != 'beta':
            print("Wrong option in get1DHistList.")
            return
        hl = []
        if opt == 'gamma':
            hl.append(self.gamma_hist)
            hl.append(self.gamma_coinc_hist)
        if opt == 'beta':
            hl.append(self.beta_hist)
            hl.append(self.beta_coinc_hist)
            hl.append(self.beta30_hist)
            hl.append(self.beta81_hist)
            hl.append(self.beta250_hist)
            hl.append(self.beta350_hist)
        return hl
    
    def show_histograms(self):
        """ Show gamma, beta, and beta-gamma histograms """
        fig = plt.figure(constrained_layout=True, figsize=[10,9])
        gs = GridSpec(3,3, figure=fig)
        ax1 = fig.add_subplot(gs[1,-1])
        ax2 = fig.add_subplot(gs[2,-1])
        ax3 = fig.add_subplot(gs[1:,:-1])
        self.gamma_coinc_hist.draw(ax1)
        self.beta_coinc_hist.draw(ax2)
        self.beta_gamma_hist.draw(fig,ax3,self.ROI_limits_chan)
        plt.show()
            
    def makeInfo(self):
        """ Make brief measurement information string"""
        txt = format("Detector: %s\n" % self.detector)
        txt += format("Type: %s Qualifier: %s\n" % (self.data_type, self.qual))
        txt += format("Calibration method: %s\n" % self.calMethod)
       # txt += format("Calibration date: %s\n" % self.calib_date.strftime("%Y-%m-%d %H:%M:%S"))
        if self.data_type == 'SAMPLEPHD' or \
            self.data_type == 'SPIKEPHD' or \
            self.data_type == 'REGENPHD':
                    txt += format("Coll. start: %s\n" % self.collstart_time.strftime("%Y-%m-%d %H:%M:%S"))
                    txt += format("Coll. stop: %s\n" % self.collstop_time.strftime("%Y-%m-%d %H:%M:%S"))
                    txt += format("Air volume: %2.2f m3\n" %self.air_volume)
                    txt += format("Xe Vol: %1.3f +/- %1.3f ml\n" 
                                  %(self.xe_vol, self.xe_vol_err))
                    txt += format("Yield: %1.2f +/- %1.2f\n" 
                                  %(self.xe_yield, self.xe_yield_err))
        txt += format("Acq. start: %s\n" % self.acq_start.strftime("%Y-%m-%d %H:%M:%S"))
        txt += format("Real: %s  Live: %s\n" % (self.acq_realtime, self.acq_livetime))
        if self.makeHist:
            txt += format("Gamma  singles CR: %1.3e\n" % (self.gammaSinglesCR/self.acq_livetime))
            txt += format("Beta  singles CR: %1.3e\n" % (self.betaSinglesCR/self.acq_livetime))
            txt += format("Beta-gamma coinc. CR: %1.3e\n" % (self.coincCR/self.acq_livetime))
        self.info = txt
        
  
            
    def IMSFormat(self):
        """Create data string on IMS format""" 
        #Assign dummy values for data not used by CUBE-systems"
        if self.stationType == 'CUBE':
            self.msg_type = 'CUBE'
            self.msg_id = "1 DUMMY"
            self.qual = 'FULL'
            self.sample_id = "SAMPLE_ID_DUMMY"
            self.detbk_id = "DETBK_ID_DUMMY"
            self.gasbk_id = "GASBK_ID_DUMMY"
            self.archive_nbr = '00'
            self.gamma_maxenergy = 999
            self.beta_maxenergy = 999
            self.nbg_gamma_maxene = 999
            self.nbg_beta_maxene = 999
            self.gEfficiency = []
            if self.data_type != 'SAMPLEPHD':
                self.srid = 'DUMMYSRID'
        #Write buffer
        buf = format("BEGIN IMS2.0\n")
        buf += format("MSG_TYPE %s\n" %self.msg_type)
        buf += format("MSG_ID %s\n" %self.msg_id)
        buf += format("DATA_TYPE %s\n" %self.data_type)
        buf += format("#Header 3\n")
        buf += format("%s %s B GEOMETRY   %s\n" %(self.station, self.detector, self.qual))
        buf += format("%s\n" %self.srid)
        buf += format("%s    %s     %s\n" %(self.sample_id, self.detbk_id, self.gasbk_id))
        date_time = self.transmit_time.strftime("%Y/%m/%d %H:%M:%S.0")
        buf += format("%s\n" %date_time)
        buf += format("#Comment\n")
        buf += format("%s\n" %self.comment)
        if self.data_type == 'SAMPLEPHD':
            buf += format("#Collection\n")
            coll_start = self.collstart_time.strftime("%Y/%m/%d %H:%M:%S.0")
            coll_stop = self.collstop_time.strftime("%Y/%m/%d %H:%M:%S.0")
            buf += format("%s %s %2.6f\n" %(coll_start, coll_stop,self.air_volume))
        buf += format("#Acquisition\n")
        acq_start = self.acq_start.strftime("%Y/%m/%d %H:%M:%S.0")
        buf += format("%s %5.6f %5.6f\n" %(acq_start, self.acq_realtime, self.acq_livetime))
        if self.data_type == 'SAMPLEPHD':
            buf += format("#Processing\n")
            buf += format("%2.5f %2.5f\n" %(self.xe_vol, self.xe_vol_err))
            buf += format("%2.5f %2.5f\n" %(self.xe_yield, self.xe_yield_err))
            buf += format("%s\n" %self.archive_nbr)
        buf += format("#Calibration\n")
        date_time = self.calib_date.strftime("%Y/%m/%d %H:%M:%S.0")
        buf += format("%s\n" %date_time)
        buf += format("#g_Energy\n")
        for e in self.gEnergy:
            buf += format("%2.6f\t%2.6f\t%2.6f\n" %(e[0], e[1], e[2]))
        buf += format("#b_Energy\n")
        for e in self.bEnergy:
            buf += format("%2.6f\tC %2.6f\t%2.6f\n" %(e[0], e[1], e[2]))
        buf += format("#g_Resolution\n")
        for e in self.gResolution:
            buf += format("%2.6f\t%2.6f\t%2.6f\n" %(e[0], e[1], e[2]))
        buf += format("#b_Resolution\n")
        for e in self.bResolution:
            buf += format("%2.6f\t%2.6f\t%2.6f\n" %(e[0], e[1], e[2]))
        buf += format("#g_Efficiency\n")
        for e in self.gEfficiency:
            buf += format("%2.6f\t%1.6f\t%1.6f\n" %(e[0], e[1], e[2]))
        buf += format("#ROI_Limits\n")
        for r in self.ROI_limits_ene:
            buf += format("%d\t%3.5f\t%3.5f\t%3.5f\t%3.5f\n" %(r[0], r[1], r[2], r[3], r[4]))
        #Also write ROIs in channels if this is a CUBE
        if self.stationType == 'CUBE':
            buf += format("#ROI_Chan_Limits\n")
            for r in self.ROI_limits_chan:
                buf += format("%d\t%d\t%d\t%d\t%d\n" %(r[0], r[1], r[2], r[3], r[4]))
        buf += format("#b-gEfficiency\n")
        for e in self.bg_eff:
            buf += format("%s\t%d\t%1.6f\t%1.6f\n" %(e[0], e[1], e[2], e[3]))
        buf += format("#Ratios\n")
        for r in self.ratio:
            buf += format("%s\t%d\t%d\t%1.6f\t%1.6f\n" %(r[0], r[1], r[2], r[3], r[4]))
        buf += format("#g_Spectrum\n")
        buf += format("%d\t%d\n" %(self.ngamma_chan, self.gamma_maxenergy))
        i = 0
        for c in range(self.ngamma_chan):
            if not c % 5: 
                buf += format("%d\t%d\t%d\t%d\t%d\t%d\n" %(i,
                        self.gamma[i], self.gamma[i+1], self.gamma[i+2], 
                        self.gamma[i+3], self.gamma[i+4]))
                
                i += 5
                if (self.ngamma_chan - i) < 5: break
        m = self.ngamma_chan - i
        if m > 0:
            buf += format("%d\t" %i)
            for j in range(m):
                buf += format("%d\t" %self.gamma[i+j])
        buf += '\n'
        buf += format("#b_Spectrum\n")
        buf += format("%d\t%d\n" %(self.nbeta_chan, self.beta_maxenergy))
        i = 0
        for c in range(self.nbeta_chan):
            if not c % 5: 
                buf += format("%d\t%d\t%d\t%d\t%d\t%d\n" %(i,
                        self.beta[i], self.beta[i+1], self.beta[i+2], 
                        self.beta[i+3], self.beta[i+4]))
                
                i += 5
                if (self.nbeta_chan - i) < 5: break
        m = self.nbeta_chan - i
        if m > 0:
            buf += format("%d\t" %i)
            for j in range(m):
                buf += format("%d\t" %self.beta[i+j])
        buf += '\n'
        buf += format("#Histogram\n")
        buf += format("%d\t%d\t%d\t%d\n" %(self.nbg_gamma_chan, 
                                               self.nbg_beta_chan, 
                                               self.nbg_gamma_maxene, 
                                               self.nbg_beta_maxene))
        i = 0
        for c in self.beta_gamma:
            if i > 0 and not (i % self.nbg_beta_chan):
                buf += '\n'
            buf += format("%d " %c)
            i += 1
        if (self.stationType != 'CUBE'):
            if not (self.status & CERTIFICATE_ERR):
                buf += format("#Certificate\n")
                buf += self.certificate
        buf += '\nSTOP'
        return buf  


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  2 15:23:37 2020

@author: ringbom
This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""

"""
Physics constants

Radioxenon decay constants and branching ratios are taken from the FOI report
"Improvement of the SAUNA Noble Gas System Calibration Procedures, FOI-R--3451--SE (July 2012)"

Radioxenon NE discrimination line parameters are taken from 
M.B.Kalinowski at al, Discrimination of Nuclear Explosions against Civilian 
Sources Based on Atmospheric XenonIsotopic Activity Ratios, Pure and Applied Geophysics 167 (2010) 517.
"""

K_S = 1.6449
XE133_LAM = 1.52956e-6         #Decay constant for 133Xe(g) (1/s)
XE131M_LAM = 6.72467e-7        #Decay constant for 131Xe(m) (1/s)
XE133M_LAM = 3.66326e-6        #Decay constant for 133Xe(m) (1/s)
XE135_LAM = 2.10657e-5         #Decay constant for 135Xe(g) (1/s)
XE135M_LAM = 7.5556e-4         #Decay constant for 135Xe(m) (1/s)
I133_LAM = 9.26e-6             #Decay constant for 133I(g) (1/s)
I131_LAM = 9.9967e-7           #Decay constant for 131I(g) (1/s)
CE141_LAM = 2.4676e-7          #Decay constant for 141Ce (1/s)
CE137_LAM = 7.3022e-10         #Decay constant for 137Cs (1/s)
BA140_LAM = 6.29085e-7         #Decay constant for 140Ba (1/s)
LA140_LAM = 4.77944e-6         #Decay constant for 140La (1/s)
BG_BR_133XE_80 = 0.37289       #Branching ratio for beta/80 keV gamma decay of 133Xe
BG_BR_133XE_30 = 0.48940       #Branching ratio for beta/30 keV X-ray decay of 133Xe
BG_BR_131MXE_30 = 0.54000      #Branching ratio for EC/30 keV X-ray decay of 131mXe
BG_BR_133MXE_30 = 0.55770      #Branching ratio for EC/30 keV X-ray decay of 133mXe
BG_BR_135XE = 0.90000          #Branching ratio for beta/250 keV gamma decay of 135Xe
BG_BR_133XE_80_ERR = 0.005     #Error for Branching ratio for beta/80 keV gamma decay of 133Xe
BG_BR_133XE_30_ERR = 0.013     #Error for Branching ratio for beta/30 keV X-ray decay of 133Xe
BG_BR_131MXE_30_ERR = 0.02     #Error for Branching ratio for EC/30 keV X-ray decay of 131mXe
BG_BR_133MXE_30_ERR = 0.013    #Error for Branching ratio for EC/30 keV X-ray decay of 133mXe */
BG_BR_135XE_ERR = 0.01         #Error for Branching ratio for beta/250 keV gamma decay of 135Xe */
XE_AIR = 0.087                 #Volume (ml) of Xe per m3 of air */
RN222_LAM = 2.0974e-6          #Radon-222 decay constant (1/s) */
BG_BR_222RN = 0.376            #Radon-222 bg branching ratio for 352 keV gamma energy*/ 
BG_BR_222RN_ERR = 0.01         #Radon-222 bg branching ratio for 352 keV gamma energy TODO: This is a generic value. Change!!! */
KAL_K0 = 1e-6                  #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133 vs Xe133m/Xe131m  separation parameter K */  
KAL_M0 = 4.4388                #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133 vs Xe133m/Xe131m  exp decay slope m */  
KAL_K1 = 2e-2                  #Kalinowski 2010 MIRC discrimination line: Xe133m/Xe133 vs Xe133m/Xe131m  separation parameter K*/  
KAL_M1 = 0.6972                #Kalinowski 2010 MIRC discrimination line: Xe133m/Xe133 vs Xe133m/Xe131m exp decay slope m */  
KAL_K2 = 3.78e-22              #Kalinowski 2010 MIRC discrimination line: Xe135/Xe131m vs Xe133/Xe131m  separation parameter K */  
KAL_M2 = 9.0774                #Kalinowski 2010 MIRC discrimination line: Xe135/Xe131m vs Xe133/Xe131m  exp decay slope m */  
KAL_K3 = 2.0e-1                #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133m vs Xe135/Xe131m  separation parameter K */  
KAL_M3 = 0.7996                #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133m vs Xe135/Xe131m  exp decay slope m */  
KAL_K4 = 3.27e+15              #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133 vs Xe133m/Xe133  separation parameter K */  
KAL_M4 = 9.7847                #Kalinowski 2010 MIRC discrimination line: Xe135/Xe133 vs Xe133m/Xe133 exp decay slope m */ 
GAMMA_ENE_RN222 = 350          #Gamma energy (keV) for Rn222 */   
GAMMA_ENE_XE135 = 249.794      #Gamma energy (keV) for Xe-135 */
GAMMA_ENE_XE133 = 80.9979      #Gamma energy (keV) for Xe-133  */
GAMMA_ENE_XE131M = 163.930     #Gamma energy (keV) for Xe-131m  */
GAMMA_ENE_XE133M = 233.221     #Gamma energy (keV) for Xe-133m  */
XRAY_ENE_XE133 = 31.61         #X-ray energy (keV) for Xe-133  */
XRAY_ENE_XE131M = 30.38        #X-ray energy (keV) for Xe-131m  */
XRAY_ENE_XE133M = 30.38        #X-ray energy (keV) for Xe-133m  */
XRAY_ENE_XE135 = 31.61         #X-ray energy (keV) for Xe-135  */
BETA_ENDPOINT_RN222 = 672          #Beta endpoint energy (keV) for Rn-222 decay in coinc. with 350 keV gamma */
BETA_ENDPOINT_XE133_GAMMA = 346.4  #Beta endpoint energy (keV) for Xe-133 decay in coinc. with 80.99979 keV gamma */
BETA_ENDPOINT_XE133_XRAY = 391.4   #Beta (+CE) endpoint energy (keV) for Xe-133 decay in coinc. with X-ray keV */
BETA_ENDPOINT_XE135 = 915          #Beta endpoint energy (keV) for Xe-135 decay in coinc. with 249.794 keV gamma */
XRAY_ENE_CS137 = 32.86             #Cs-137 K X-ray energy (keV) */
GAMMA1_ENE_CS137 = 661.657         #Cs-137 primary gamma energy (keV) */
XRAY_ENE_EU154 = 43.91             #E-154 K X-ray energy (keV) */
GAMMA1_ENE_EU154 = 123.0706        #E-154 Primary gamma energy (keV) */
GAMMA2_ENE_EU154 = 247.9290        #E-154 Second gamma energy (keV) */
GAMMA3_ENE_EU154 = 591.755         #E-154 Third gamma energy (keV) */
GAMMA4_ENE_EU154 = 692.4205        #E-154 Fourth gamma energy (keV) */
XRAY_ENE_EU152 = 40.96             #E-152 K X-ray energy (keV) */
GAMMA1_ENE_EU152 = 121.7817        #E-152 Primary gamma energy (keV) */
GAMMA2_ENE_EU152 = 344.2785        #E-152 Second gamma energy (keV) */
GAMMA3_ENE_EU152 = 244.6975        #E-152 Third gamma energy (keV) */
GAMMA4_ENE_EU152 = 411.1163        #E-152 Fourth gamma energy (keV) */
XRAY_ENE_AM241 = 13.9              #Am-241 L X-ray energy (keV) */
GAMMA1_ENE_AM241 = 59.5409         #Am-241 Primary gamma energy (keV) */
GAMMA2_ENE_AM241 = 26.3446         #Am-241 Second gamma energy (keV) */
K_CE_ENE_XE131M = 129.37            #K - Conversion electron energy (keV) for Xe-131m  */
L_CE_ENE_XE131M = 158.79            # L - Conversion electron energy (keV) for Xe-131m  */
M_CE_ENE_XE131M = 162.89            #M - Conversion electron energy (keV) for Xe-131m  */
N_CE_ENE_XE131M = 163.74            #N - Conversion electron energy (keV) for Xe-131m  */
O_CE_ENE_XE131M = 163.91            #O - Conversion electron energy (keV) for Xe-131m  */
K_CE_ENE_XE131M_INT = 0.616         #K - Conversion electron intensity for Xe-131m  */
L_CE_ENE_XE131M_INT = 0.288         #L - Conversion electron intensity for Xe-131m  */
M_CE_ENE_XE131M_INT = 0.0659        #M - Conversion electron intensity for Xe-131m  */
N_CE_ENE_XE131M_INT = 0.01342       #N - Conversion electron intensity for Xe-131m  */
O_CE_ENE_XE131M_INT = 0.00147       #O - Conversion electron intensity for Xe-131m  */
K_CE_ENE_XE133M = 198.660           #K - Conversion electron energy (keV) for Xe-133m  */
L_CE_ENE_XE133M = 227.768           #L - Conversion electron energy (keV) for Xe-133m  */
M_CE_ENE_XE133M = 232.079           #M - Conversion electron energy (keV) for Xe-133m  */
N_CE_ENE_XE133M = 233.013           #N - Conversion electron energy (keV) for Xe-133m  */
O_CE_ENE_XE133M = 233.205           #O - Conversion electron energy (keV) for Xe-133m  */
K_CE_ENE_XE133M_INT = 0.629         #K - Conversion electron intensity for Xe-133m  */
L_CE_ENE_XE133M_INT = 0.210         #L - Conversion electron intensity for Xe-133m  */
M_CE_ENE_XE133M_INT = 0.047         #M - Conversion electron intensity for Xe-133m  */
N_CE_ENE_XE133M_INT = 0.0096        #N - Conversion electron intensity for Xe-133m  */
O_CE_ENE_XE133M_INT = 0.00107       #O - Conversion electron intensity for Xe-133m  */


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr  7 15:44:34 2020

@author: ringbom

This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
import wx

# replace with wx.NewIdRef(count=1) 

OPEN_SAMPLE = wx.NewIdRef(count=1) 
OPEN_GASBK =  wx.NewIdRef(count=1) 
OPEN_DETBK =  wx.NewIdRef(count=1) 
OPEN_QC =  wx.NewIdRef(count=1) 
OPEN_CALIB =  wx.NewIdRef(count=1) 
PRINT_SAMPLE = wx.NewIdRef(count=1) 
PRINT_GASBK =  wx.NewIdRef(count=1) 
PRINT_DETBK = wx.NewIdRef(count=1) 
PRINT_QC = wx.NewIdRef(count=1) 
PRINT_CALIB =  wx.NewIdRef(count=1) 
OPEN_MEAS =  wx.NewIdRef(count=1) 
OPEN_DATASET =  wx.NewIdRef(count=1) 
SAVE_MEAS =  wx.NewIdRef(count=1) 
SAVE_DATASET =  wx.NewIdRef(count=1) 
SAVE_SAMPLE_PHD =  wx.NewIdRef(count=1) 
SAVE_GASBK_PHD =  wx.NewIdRef(count=1) 
SAVE_DETBK_PHD =  wx.NewIdRef(count=1) 
SAVE_QC_PHD =  wx.NewIdRef(count=1) 
SAVE_CALIB_PHD =  wx.NewIdRef(count=1) 
SAVE_CSV = wx.NewIdRef(count=1) 
OPTIONS = wx.NewIdRef(count=1) 
ANALYZE = wx.NewIdRef(count=1) 
CLEAR_MEAS =  wx.NewIdRef(count=1) 
CALC_MSET_FILE =  wx.NewIdRef(count=1) 
CLEAR_MSET = wx.NewIdRef(count=1) 
PLOT_MSET =  wx.NewIdRef(count=1) 
APPEND_MSET =  wx.NewIdRef(count=1) 
COMPARE_MSET =  wx.NewIdRef(count=1) 
MSET_REPORT = wx.NewIdRef(count=1) 
MSET_FREQUENCY = wx.NewIdRef(count=1) 
MSET_MIRC3 = wx.NewIdRef(count=1) 
MSET_MIRC4 = wx.NewIdRef(count=1) 
MSET_UNMARK = wx.NewIdRef(count=1) 
PLOT_XEVOL = wx.NewIdRef(count=1) 
PLOT_XEYIELD = wx.NewIdRef(count=1) 
PLOT_AIRVOL = wx.NewIdRef(count=1) 
PLOT_ERR = wx.NewIdRef(count=1) 
HELP = wx.NewIdRef(count=1) 
ABOUT = wx.NewIdRef(count=1) 
CLEAR_LOG = wx.NewIdRef(count=1) 
SAVE_MEAS_REPORT = wx.NewIdRef(count=1) 
SAVE_MEASSET_REPORT = wx.NewIdRef(count=1) 

class menu(wx.MenuBar):
    """Menu for the GUI 'openSpex'"""
    def __init__(self):
        wx.MenuBar.__init__(self)
        #File menu
        file = wx.Menu()
        Open = wx.Menu()
        self.openMeasurement = Open.Append(OPEN_MEAS, "Measurement (.pkl)")
        self.openDataset = Open.Append(OPEN_DATASET, "Measurement set (.pkl)")
        OpenPHD =wx.Menu()
        self.openSample = OpenPHD.Append(OPEN_SAMPLE, "Sample PHD")
        self.openGasbk = OpenPHD.Append(OPEN_GASBK, "Gasbk PHD")
        self.openDetbk = OpenPHD.Append(OPEN_DETBK, "Detbk PHD")
        self.openQC = OpenPHD.Append(OPEN_QC, "QC PHD")
        self.openCalib = OpenPHD.Append(OPEN_CALIB, "Calib PHD")
        Open.AppendSubMenu(OpenPHD, "PHD")
        file.AppendSubMenu(Open, "Open")
        Save = wx.Menu()
        self.saveMeasurement = Save.Append(SAVE_MEAS, "Measurement (.pkl)")
        self.saveDataset = Save.Append(SAVE_DATASET, "Measurement set (.pkl)")
        self.saveMeasurementReport = Save.Append(SAVE_MEAS_REPORT, "Measurement report")
        self.saveMeasurementsetReport = Save.Append(SAVE_MEASSET_REPORT, "Measurementset report")
        file.AppendSubMenu(Save, "Save")
        
        Export = wx.Menu()
        self.saveDatasetCSV = Export.Append(SAVE_CSV, "Measurement set (.csv)")
        SavePHD = wx.Menu()
        self.saveSamplePHD = SavePHD.Append(SAVE_SAMPLE_PHD, "Sample")
        self.saveGasbkPHD = SavePHD.Append(SAVE_GASBK_PHD, "Gasbk")
        self.saveDetbkPHD = SavePHD.Append(SAVE_DETBK_PHD, "Detbk")
        self.saveQCPHD = SavePHD.Append(SAVE_QC_PHD, "QC")
        self.saveCalibPHD = SavePHD.Append(SAVE_CALIB_PHD, "Calib")
        Export.AppendSubMenu(SavePHD,"PHD")
        file.AppendSubMenu(Export, "Export")
        
        self.clearLog = file.Append(CLEAR_LOG, "Clear Log")
        self.quit = file.Append(wx.ID_EXIT, "Quit\tCtrl+W")
        self.Append(file,"File")
        #Measurement menu
        Mea = wx.Menu()
        self.analyze = Mea.Append(ANALYZE, "BGM analysis")
        PrintPHD =wx.Menu()
        self.printSample = PrintPHD.Append(PRINT_SAMPLE, "Sample")
        self.printGasbk = PrintPHD.Append(PRINT_GASBK, "Gasbk")
        self.printDetbk = PrintPHD.Append(PRINT_DETBK, "Detbk")
        self.printQC = PrintPHD.Append(PRINT_QC, "QC")
        self.printCalib = PrintPHD.Append(PRINT_CALIB, "Calib")
        Mea.AppendSubMenu(PrintPHD, "Print PHD")
        Mea.AppendSeparator()
        self.clearMea = Mea.Append(CLEAR_MEAS, "Clear")
        self.Append(Mea,"Measurement")
        #Measurementset
        MeaSet = wx.Menu()
        self.calcMsetFile = MeaSet.Append(CALC_MSET_FILE, "Calculate")
        self.reportMset = MeaSet.Append(MSET_REPORT, "Report")
        Mea.AppendSeparator()
        self.plotMset = MeaSet.Append(PLOT_MSET, "Timeseries")
        self.appendMset = MeaSet.Append(APPEND_MSET, "Append")
        self.freqMset = MeaSet.Append(MSET_FREQUENCY, "Frequency")
        self.mirc3Mset = MeaSet.Append(MSET_MIRC3, "MIRC3")
        self.mirc4Mset = MeaSet.Append(MSET_MIRC4, "MIRC4")
        PlotParam =wx.Menu()
        self.plotXeVol = PlotParam.AppendCheckItem(PLOT_XEVOL, "Xenon volume")
        self.plotXeYield = PlotParam.AppendCheckItem(PLOT_XEYIELD, "Xenon Yield")
        self.plotAirVol = PlotParam.AppendCheckItem(PLOT_AIRVOL, "Air volume")
        PlotParam.AppendSeparator()
        self.plotErrors = PlotParam.AppendCheckItem(PLOT_ERR, "Show errors")
        MeaSet.AppendSubMenu(PlotParam, "Plot parameter")
        MeaSet.AppendSeparator()
        self.unmarkMset = MeaSet.Append(MSET_UNMARK, "Unmark all")
        self.clearMset = MeaSet.Append(CLEAR_MSET, "Clear")
        self.Append(MeaSet,"Measurementset")
        #Options menu
        Options = wx.Menu()
        self.options = Options.Append(OPTIONS, "Options")
        self.Append(Options,"Options")
        Help = wx.Menu()
        self.help = Help.Append(HELP, "Help")
        self.about = Help.Append(ABOUT, "About")
        self.Append(Help,"Help")


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Apr  1 21:41:15 2020

@author: ringbom

This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
from matplotlib import pyplot as plt
from matplotlib.gridspec import GridSpec
import os
import numpy as np

from open_spex import BGMeasurement as bg
from open_spex import bgm as bgm



font = 'Liberation Mono'
if os.name == 'nt':
    font = 'Courier New'

class BGSample:
    """
    A beta-gamma sample with sample, gas background (optional), 
    detector background, and qc (optional).
    Created either by using BGMeasurement objects or filenames.
    
    Parameters
    ----------
    sample: BGMeasurement or filename string
    detbk:  BGMeasurement or filename string 
    gasbk:  BGMeasurement or filename string (optional) 
    qc:     BGMeasurement or filename string (optional) 
    """
    def __init__(self, sample, detbk, gasbk= None, qc = None):
        
        if isinstance(sample, str):
            self.sample = bg.BGMeasurement(sample)
        else:
            self.sample = sample
        if isinstance(detbk, str):
            self.detbk = bg.BGMeasurement(detbk)
        else:
            self.detbk = detbk
        if isinstance(gasbk, str):
            self.gasbk = bg.BGMeasurement(gasbk)
        else:
            self.gasbk = gasbk
        if isinstance(qc, str):
            self.qc = bg.BGMeasurement(qc)
        else:
            self.qc = qc
        if gasbk != None:
            self.tau = (self.sample.acq_start - self.gasbk.acq_start).total_seconds()
        else:
            self.tau = 0
        if hasattr(self.sample,"collstart_time"):
            self.collTime = (self.sample.collstop_time - self.sample.collstart_time).total_seconds()
            self.procTime = (self.sample.acq_start - self.sample.collstop_time).total_seconds()
        else:
            self.collTime = None
            self.procTime = None
            print("Sample has no collection info. Only activities can be calculated.")
        #self.check()
        self.bgmHandler = None
        self.analysisInfo = "Not analyzed"
        self.stationType = self.sample.stationType
        self.makeInfo()
            
    def bgmAnalysis(self, combine = True, verbose = False, ingrowth = True, 
                    unit = 0.001, optROI = False):
        """
        Do a BGM-analysis.
        Parameters:
        ----------
        combine             True (default)/False. Use different combinations of ROI7-10 in the Xe-133 calculation 
                            depending on detection pattern. If 'False', use original BGM. 
        ingrowth            True (default)/False. Correct for Xe-133m->Xe133 ingrowth if Xe133m detected.
        verbose             True/False (default). More printout.
        unit                1 = Bq or 0.001 (default) = mBq
        """
        self.bgmHandler = bgm.bgmHandler(self, verbose)
        bgm.bgm(self.bgmHandler, combine, unit=unit, ingrowth = ingrowth)
        self.makeAnalysisInfo()
    
    def bgmAnalysis3(self, verbose = False, unit = 0.001):
        """This is for testing only. Does a BGM - analysis without using the BGSample object.""" 
        if self.gasbk != None:
            gAcqLive = self.gasbk.acq_livetime
            gAcqReal = self.gasbk.acq_realtime
        else:
            gAcqLive = None
            gAcqReal = None
        tau =   self.tau
        collTime = self.collTime
        procTime = self.procTime
        sAcqReal = self.sample.acq_realtime
        sAcqLive = self.sample.acq_livetime
        dAcqReal = self.detbk.acq_realtime
        dAcqLive = self.detbk.acq_livetime
        xeVol = self.sample.xe_vol
        xeVolErr = self.sample.xe_vol_err
        eff = np.zeros(18)
        i = 0
        for e in self.sample.bg_eff:
            eff[i] = e[2]
            eff[i+9] = e[3]
            i += 1
        ratio = np.zeros(32)
        i = 0
        for r in self.sample.ratio:
            ratio[i] = r[3]
            ratio[i+16] = r[4]
            i += 1
        sampleGross = self.sample.roi_counts
        detbkGross = self.detbk.roi_counts
        if self.gasbk != None:
            gasbkGross = self.gasbk.roi_counts
        else:
            gasbkGross = None
        setup = [tau, collTime, procTime, sAcqReal, sAcqLive, gAcqReal, 
                 gAcqLive, dAcqReal, dAcqLive, xeVol, xeVolErr]
        print("setup = " , setup)
        print("eff = ", eff)
        print("ratio = ", ratio)
        print("sample gross = ", sampleGross)
        print("gasbk gross = ", gasbkGross)
        print("detbk gross = ", detbkGross)
        self.bgmHandler = bgm.bgmHandler(None, verbose, setup, eff, ratio, 
                                    sampleGross, gasbkGross, detbkGross)
        bgm.bgm(self.bgmHandler, unit=unit)
        
    def makeInfo(self):
        """Make a string with some collection information."""
        txt = format("Station: %s\n" % self.sample.station)
        txt = format("Detector: %s\n" % self.sample.detector)
        if hasattr(self.sample, "collstart_time"):
            txt = format("Coll. start: %s\n" % self.sample.collstart_time)
            txt = format("Coll. stop: %s\n" % self.sample.collstop_time)
        self.info = txt
        
    def makeAnalysisInfo(self):
        """Make a string with essential analysis results"""
        if self.bgmHandler.inputErrors:
            self.analysisInfo = "Input errors. No analysis performed."
            return
        txt = format("Analysis method: %s\n\n" %self.bgmHandler.analysisMethod)
        if not self.bgmHandler.activityOnly:
            ac =  self.bgmHandler.AC
            ace =  self.bgmHandler.ACErr
            lc =  self.bgmHandler.LC
            txt += '{:<10s}{:<17s}{:<7s}{:<7s}\n'.format('','AC [mBq/m3]',' LC','Detected') 
            txt += '------------------------------------------\n'
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-133:',self.bgmHandler.Xe133AC,'+/-',
                            self.bgmHandler.Xe133ACErr, self.bgmHandler.Xe133ACLC, self.bgmHandler.xe133Det)   
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-131m:',ac[4],'+/-',ace[4], lc[4], self.bgmHandler.xe131mDet)
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-133m:',ac[5],'+/-',ace[5], lc[5], self.bgmHandler.xe133mDet) 
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-135:',ac[1],'+/-',ace[1], lc[1], self.bgmHandler.xe135Det)     
            txt += '------------------------------------------\n'
        else:
            a =  self.bgmHandler.A
            ae =  self.bgmHandler.AErr
            lc =  self.bgmHandler.LCA
            txt += '{:<10s}{:<17s}{:<7s}{:<7s}\n'.format('','A [mBq]',' LC', 'Detected') 
            txt += '------------------------------------------\n'
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-133:',self.bgmHandler.Xe133A,'+/-',
                            self.bgmHandler.Xe133AErr, self.bgmHandler.Xe133ALC, self.bgmHandler.xe133Det)   
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-131m:',a[4],'+/-',ae[4], lc[4], self.bgmHandler.xe131mDet)
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-133m:',a[5],'+/-',ae[5], lc[5], self.bgmHandler.xe133mDet) 
            txt += '{:<10s}{:<7.3f}{:<4s}{:<7.3f}{:<7.3f}{:<7d}\n'.format('Xe-135:',a[1],'+/-',ae[1], lc[1], self.bgmHandler.xe135Det)     
            txt += '------------------------------------------\n'
        self.analysisInfo = txt
        
    def plot(self):
        """Plot sample histograms and analysis info"""
        fig = plt.figure(constrained_layout=True, figsize=[10,9])
        gs = GridSpec(2,2, figure=fig)
        ax1 = fig.add_subplot(gs[0,0])
        ax2 = fig.add_subplot(gs[1,0])
        ax4 = fig.add_subplot(gs[0,1])
        ax3 = fig.add_subplot(gs[1,1])
        self.sample.gamma_hist.draw(ax1)
        self.sample.beta_hist.draw(ax2)
        self.sample.drawBGHist(fig,ax3)
        txt = self.info + "\n" + self.analysisInfo
        font_dict = {'family': font, 'color':'black', 'size':12}
        ax4.axis('off')
        ax4.text(0., 0.95, txt, va='top', fontdict = font_dict)
        plt.show()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May  4 08:02:41 2020

@author: ringbom
This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
import wx
import pickle
from os import path

def exportIMSFile(bgmeas,pathname):
    fn = path.join(pathname, bgmeas.phdFileName)
    try:
        with open(fn, 'w') as file:
            file.write(bgmeas.IMSFormat())      
    except IOError:
        wx.LogError("Cannot save current data in file '%s'." % fn)
    print("Exported data to file %s" %fn)
        
def ReadFile(parent,ext=[],descr=[]):
    """
    Dialogue for opening and reading a file.
    Parameters:
    ----------
    ext:      List of extension strings [ext1,ext2,...]
    descr     List of corresponding description strings [descr1,descr2, ...]
    
    Returns pathname, data
    Returns 'None', 'None' if cancelled.
    Returns pathname, 'None' if file could not be opened. 
    """
    wildc = ""
    i = 0
    for ex in ext:
        if i > 0:
            wildc += "|"
        wildc +=descr[i] + " (*." + ex +")|*." + ex
        i +=1
    
    with wx.FileDialog(parent, "Open file", 
                       wildcard = wildc,
                       style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as fileDialog:

        if fileDialog.ShowModal() == wx.ID_CANCEL:
            return None, None          
        pathname = fileDialog.GetPath()
        try:
            with open(pathname, 'r') as file:
                print("Reading file %s" % pathname)
                data = file.read()
        except IOError:
            wx.LogError("Cannot open file %s\n" %pathname)
            data = None
    return pathname, data

def SaveFile(parent,buf,ext=[],descr=[]):
    """ 
    Dialog saving content in string 'buf' as file
    Parameters:
    ----------
    buf       Data string
    ext:      List of extension strings [ext1,ext2,...]
    descr     List of corresponding description strings [descr1,descr2, ...]
    """
    wildc = ""
    i = 0
    for ex in ext:
        if i > 0:
            wildc += "|"
        wildc +=descr[i] + " (*." + ex +")|*." + ex
        i +=1
    with wx.FileDialog(parent, "Save file", wildcard=wildc,
                       style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT) as fileDialog:

        if fileDialog.ShowModal() == wx.ID_CANCEL:
            return
        pathname = fileDialog.GetPath()
        try:
            with open(pathname, 'w') as file:
              file.write(buf)      
        except IOError:
            wx.LogError("Cannot save current data in file '%s'." % pathname)
            
    
def SavePickleFile(parent, obj):
    """
    Dialogue for saving an object to a pickle file.
    Returns 'True' if file was saved; 'False' otherwize.
    """
    with wx.FileDialog(parent, "Save pkl-file", 
                       wildcard="pkl files (*.pkl)|*.pkl",
                       style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT) as fileDialog:

        if fileDialog.ShowModal() == wx.ID_CANCEL:
            return False
        pathname = fileDialog.GetPath()
        try:
            with open(pathname, 'wb') as file:
               pickle.dump(obj, file)
        except IOError:
            wx.LogError("Cannot save current data in file %s\n" %pathname)
            return False
        return True
            

def ReadPickleFile(parent):
    """
    Dialogue for opening and reading an object from a pickle file. 
    Returns pathname, object. 
    Returns 'None', 'None' if cancelled.
    Returns pathname, 'None' if file could not be opened. 
    """
    with wx.FileDialog(parent, "Open pkl file", 
                       wildcard="pkl files (*.pkl)|*.pkl",
                       style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as fileDialog:

        if fileDialog.ShowModal() == wx.ID_CANCEL:
            return None, None   
        pathname = fileDialog.GetPath()
        try:
            with open(pathname, 'rb') as file:
                print("Reading file %s" % pathname)
                obj = pickle.load(file)
        except IOError:
            wx.LogError("Cannot open file %s\n" %pathname)
            obj = None
    return pathname, obj


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr  7 15:29:46 2020

@author: ringbom

This file is part of 'openSpex'
Copyright (c) 2021 Swedish Defence Research Agency (FOI)
"""
import wx
import wx.lib.newevent
from pathlib import Path
from os import path
import os

SaveEvent, EVT_OPTIONS_SAVED = wx.lib.newevent.NewEvent()

class options():
    """Options for the GUI 'openSpex'"""
    def __init__(self):
        self.ignoreFiletype = False
        self.histScaleCountrate = True
        self.verbose = False
        self.useMarkers = True
        self.loadMarkedSample = True
        self.hideMarked = False
        self.fileStationType = 'SAUNA'
        self.analysisUseGB = False
        self.combine = True
        self.originalBGM = False
        self.ingrowthCorr = True
        self.tweakParameters = [False,0,False,1,False,0,False,1]
        self.mSetPlot = ['AC', 'E', 'G']
        self.mSetPlotIso = ['Xe-133', 'Xe-131m', 'Xe-133m','Xe-135']
        self.writeResultFiles = True
        self.moveDataFiles = False
        self.spexFHome = path.join(Path.home(),".openSpex")
        if not path.exists(self.spexFHome):
            os.mkdir(self.spexFHome)
        self.pathFile = path.join(self.spexFHome,"DataPaths")
        try:
            with open(self.pathFile, 'r') as file:
                lines = file.read().splitlines()
                self.dataPath = lines[0]
                self.detbkPath = lines[1]
                self.resultPath = lines[2]
        except IOError:
            self.dataPath = str(os.getcwd())
            self.detbkPath = str(os.getcwd())
            self.resultPath = str(os.getcwd())
            
class optionsPanel(wx.Frame):
    """Options window for the GUI 'openSpex' """
    def __init__(self, parent, title, options):
        wx.Frame.__init__(self, parent, title=title, 
                          size=(600,600), pos = (100,50))
        self.parent = parent
        self.options = options
        vbox = wx.BoxSizer(wx.VERTICAL)
        box1 = wx.BoxSizer()
        self.nb = wx.Notebook(self)
        #General options
        self.tab_general = wx.Panel(self.nb)
        self.cb_ignoreType = wx.CheckBox(self.tab_general,
                        label = "Ignore PHD datatype", pos = (10,20))
        self.cb_verbose = wx.CheckBox(self.tab_general, label = "Verbose", pos = (10,40))
        self.cb_useMarkers = wx.CheckBox(self.tab_general, label = "Use markers", pos = (10,60))
        self.cb_loadMarkedSample = wx.CheckBox(self.tab_general, label = "Load marked sample", pos = (10,80))
        self.cb_hideMarkedSample = wx.CheckBox(self.tab_general, label = "Hide marked samples", pos = (10,100))
        
        lb = ["Counts/bin", "Counts/bin/s"]
        self.rb_hist = wx.RadioBox(self.tab_general, label= "1D Histogram scale", 
                                   pos = (10,130), 
                                   choices= lb, majorDimension = 1, 
                                   style = wx.RA_SPECIFY_ROWS)
        wx.StaticText(self.tab_general, label="Data path:        ", pos = (10,200))
        self.dpathPicker = wx.DirPickerCtrl(self.tab_general, wx.DIRP_DIR_MUST_EXIST, 
                            path = str(Path.home()), pos=(100,200), size=(300,28))
        self.cb_moveData = wx.CheckBox(self.tab_general, label = "Move", pos = (420,200))
        wx.StaticText(self.tab_general, label="Detbk path:        ", pos = (10,240))
        self.dbkpathPicker = wx.DirPickerCtrl(self.tab_general, wx.DIRP_DIR_MUST_EXIST, 
                            path = str(Path.home()), pos=(100,240), size=(300,28))
        wx.StaticText(self.tab_general, label="Result path:        ", pos = (10,280))
        self.resultpathPicker = wx.DirPickerCtrl(self.tab_general, wx.DIRP_DIR_MUST_EXIST, 
                            path = str(Path.home()), pos=(100,280), size=(300,28))
        self.cb_writeResults = wx.CheckBox(self.tab_general, label = "Save", pos = (420,280))
        
        #analysis options
        self.tab_analysis = wx.Panel(self.nb)
        self.cb_useGB = wx.CheckBox(self.tab_analysis, label = "Use gas background", pos = (10,20))
        lb3 = ["Use ROI 7-10", "Original"]
        self.rb_nrois = wx.RadioBox(self.tab_analysis, label= "Method:", 
                                    pos = (10,70), 
                                    choices= lb3, majorDimension = 1, 
                                    style = wx.RA_SPECIFY_ROWS)
        self.cb_ingrCorr = wx.CheckBox(self.tab_analysis, label = "Correct for Xe-133m ingrowth", pos = (10,140))
        self.cb_offsetBeta = wx.CheckBox(self.tab_analysis,label = "Beta offset (chan)", pos = (10,190))
        self.sc_offsetBeta = wx.SpinCtrlDouble(self.tab_analysis, pos = (10,210), min = 0., max = 256, inc = 1.)
        self.cb_tweakBeta = wx.CheckBox(self.tab_analysis,label = "Beta tweak", pos = (10,240))
        self.sc_tweakBeta = wx.SpinCtrlDouble(self.tab_analysis, pos = (10,260), min = 0., max = 2., inc = 0.01)
        self.cb_offsetGamma = wx.CheckBox(self.tab_analysis,label = "Gamma offset (chan)", pos = (10,290))
        self.sc_offsetGamma = wx.SpinCtrlDouble(self.tab_analysis, pos = (10,310), min = 0., max = 256, inc = 1.)
        self.cb_tweakGamma = wx.CheckBox(self.tab_analysis,label = "Gamma tweak", pos = (10,340))
        self.sc_tweakGamma = wx.SpinCtrlDouble(self.tab_analysis, pos = (10,360), min = 0., max = 2., inc = 0.01)
        
        
        #Measurementset plot options
        self.tab_mset = wx.Panel(self.nb)
        self.cb_AC = wx.CheckBox(self.tab_mset,
                        label = "Activity Concentration (AC)", pos = (10,20))
        self.cb_LC = wx.CheckBox(self.tab_mset,
                        label = "Activity Concentration Critical Limit (LC)", pos = (10,40))
        self.cb_MDC = wx.CheckBox(self.tab_mset,
                        label = "MDC", pos = (10,60))
        self.cb_A = wx.CheckBox(self.tab_mset,
                        label = "Activity (A)", pos = (10,80))
        self.cb_LCA = wx.CheckBox(self.tab_mset,
                        label = "Activity Critical Limit (LCA)", pos = (10,100))
        self.cb_MDA = wx.CheckBox(self.tab_mset,
                        label = "MDA", pos = (10,120))
        self.cb_err = wx.CheckBox(self.tab_mset,
                        label = "Error bars", pos = (10,150))
        self.cb_line = wx.CheckBox(self.tab_mset,
                        label = "AC and A as line", pos = (10,170))
        self.cb_dots = wx.CheckBox(self.tab_mset,
                        label = "LC, MDC, LCA, and MDA as dots", pos = (10,190))
        self.cb_legend = wx.CheckBox(self.tab_mset,
                        label = "show legend", pos = (10,210))
        self.cb_xe133 = wx.CheckBox(self.tab_mset,
                        label = "Xe-133", pos = (10,240))
        self.cb_xe131m = wx.CheckBox(self.tab_mset,
                        label = "Xe-131m", pos = (10,260))
        self.cb_xe133m = wx.CheckBox(self.tab_mset,
                        label = "Xe-133m", pos = (10,280))
        self.cb_xe135 = wx.CheckBox(self.tab_mset,
                        label = "Xe-135", pos = (10,300))
        self.cb_rn222 = wx.CheckBox(self.tab_mset,
                        label = "Rn-222", pos = (10,320))
        
    
        self.nb.AddPage(self.tab_general, "General")
        self.nb.AddPage(self.tab_analysis, "BGM Analysis")
        self.nb.AddPage(self.tab_mset, "Measurementset plot")
        box1.Add(self.nb,1, wx.EXPAND)
        vbox.Add(box1, 6, flag = wx.EXPAND )
        vbox.Add((-1,10))
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        self.b_save = wx.Button(self,label="Save", size = (70,30))
        self.b_save.Bind(wx.EVT_BUTTON, self.OnSave)
        hbox2.Add(self.b_save)
        self.b_close = wx.Button(self,label="Close", size= (70,30))
        self.b_close.Bind(wx.EVT_BUTTON, self.OnClose)
        hbox2.Add(self.b_close, flag = wx.LEFT |wx.BOTTOM, border = 5)
        vbox.Add(hbox2, 1, wx.ALIGN_RIGHT|wx.RIGHT|wx.BOTTOM, border = 10)
        self.SetSizer(vbox)
        self.setOptions()
        self.change = False
    
    def setOptions(self):
        """Set stored options in panel"""
        self.cb_ignoreType.SetValue(self.options.ignoreFiletype)
        self.cb_verbose.SetValue(self.options.verbose)
        self.cb_useGB.SetValue(self.options.analysisUseGB)
        self.cb_ingrCorr.SetValue(self.options.ingrowthCorr)
        self.dpathPicker.SetPath(self.options.dataPath)
        self.dbkpathPicker.SetPath(self.options.detbkPath)
        self.resultpathPicker.SetPath(self.options.resultPath)
        self.cb_writeResults.SetValue(self.options.writeResultFiles)
        self.cb_moveData.SetValue(self.options.moveDataFiles)
        
        self.cb_offsetBeta.SetValue(self.options.tweakParameters[0])
        self.sc_offsetBeta.SetValue(self.options.tweakParameters[1])
        self.cb_tweakBeta.SetValue(self.options.tweakParameters[2])
        self.sc_tweakBeta.SetValue(self.options.tweakParameters[3])
        self.cb_offsetGamma.SetValue(self.options.tweakParameters[4])
        self.sc_offsetGamma.SetValue(self.options.tweakParameters[5])
        self.cb_tweakGamma.SetValue(self.options.tweakParameters[6])
        self.sc_tweakGamma.SetValue(self.options.tweakParameters[7])
        
        self.cb_useMarkers.SetValue(self.options.useMarkers)
        self.cb_loadMarkedSample.SetValue(self.options.loadMarkedSample)
        self.cb_hideMarkedSample.SetValue(self.options.hideMarked)

        
        if self.options.histScaleCountrate:
            self.rb_hist.SetSelection(1)
        else:
            self.rb_hist.SetSelection(0)
            
        if self.options.combine and not self.options.originalBGM:
            self.rb_nrois.SetSelection(0)
        if self.options.originalBGM and not self.options.combine:
            self.rb_nrois.SetSelection(1)
        if 'AC' in self.options.mSetPlot:
            self.cb_AC.SetValue(True)
        else:
            self.cb_AC.SetValue(False)
        if 'LC' in self.options.mSetPlot:
            self.cb_LC.SetValue(True)
        else:
            self.cb_LC.SetValue(False)
        if 'MDC' in self.options.mSetPlot:
            self.cb_MDC.SetValue(True)
        else:
            self.cb_MDC.SetValue(False)
        if 'A' in self.options.mSetPlot:
            self.cb_A.SetValue(True)
        else:
            self.cb_A.SetValue(False)
        if 'LCA' in self.options.mSetPlot:
            self.cb_LCA.SetValue(True)
        else:
            self.cb_LCA.SetValue(False)
        if 'MDA' in self.options.mSetPlot:
            self.cb_MDA.SetValue(True)
        else:
            self.cb_MDA.SetValue(False)
        if 'E' in self.options.mSetPlot:
            self.cb_err.SetValue(True)
        else:
            self.cb_err.SetValue(False)
        if 'L' in self.options.mSetPlot:
            self.cb_line.SetValue(True)
        else:
            self.cb_line.SetValue(False)
        if 'P' in self.options.mSetPlot:
            self.cb_dots.SetValue(True)
        else:
            self.cb_dots.SetValue(False)
        if 'G' in self.options.mSetPlot:
            self.cb_legend.SetValue(True)
        else:
            self.cb_legend.SetValue(False)
            
        if 'Xe-133' in self.options.mSetPlotIso:
            self.cb_xe133.SetValue(True)
        else:
            self.cb_xe133.SetValue(False)
        if 'Xe-131m' in self.options.mSetPlotIso:
            self.cb_xe131m.SetValue(True)
        else:
            self.cb_xe131m.SetValue(False)
        if 'Xe-133m' in self.options.mSetPlotIso:
            self.cb_xe133m.SetValue(True)
        else:
            self.cb_xe133m.SetValue(False)
        if 'Xe-135' in self.options.mSetPlotIso:
            self.cb_xe135.SetValue(True)
        else:
            self.cb_xe135.SetValue(False) 
        if 'Rn-222' in self.options.mSetPlotIso:
            self.cb_rn222.SetValue(True)
        else:
            self.cb_rn222.SetValue(False)      
        self.Layout()
        
    def OnSave(self,event):
        """Store selected options"""
        self.change = False
        self.options.ignoreFiletype = self.cb_ignoreType.GetValue()
        self.options.verbose = self.cb_verbose.GetValue()
        self.options.analysisUseGB = self.cb_useGB.GetValue()
        self.options.ingrowthCorr =  self.cb_ingrCorr.GetValue()
        self.options.useMarkers = self.cb_useMarkers.GetValue()
        self.options.loadMarkedSample = self.cb_loadMarkedSample.GetValue()
        self.options.hideMarked = self.cb_hideMarkedSample.GetValue()
        self.options.dataPath = self.dpathPicker.GetPath()
        self.options.detbkPath = self.dbkpathPicker.GetPath()
        self.options.resultPath = self.resultpathPicker.GetPath()
        self.options.writeResultFiles = self.cb_writeResults.GetValue()
        self.options.moveDataFiles = self.cb_moveData.GetValue()

        if self.rb_hist.GetSelection() ==0:
            self.options.histScaleCountrate = False
        else:
            self.options.histScaleCountrate = True
            
        if self.rb_nrois.GetSelection() ==0:
            self.options.combine = True
            self.options.originalBGM = False
        else:
            self.options.combine = False
            self.options.originalBGM = True
               
        self.options.tweakParameters[0] = self.cb_offsetBeta.GetValue()
        self.options.tweakParameters[1] = self.sc_offsetBeta.GetValue()
        self.options.tweakParameters[2] = self.cb_tweakBeta.GetValue()
        self.options.tweakParameters[3] = self.sc_tweakBeta.GetValue()
        self.options.tweakParameters[4] = self.cb_offsetGamma.GetValue()
        self.options.tweakParameters[5] = self.sc_offsetGamma.GetValue()
        self.options.tweakParameters[6] = self.cb_tweakGamma.GetValue()
        self.options.tweakParameters[7] = self.sc_tweakGamma.GetValue()
            
        self.options.mSetPlot = []
        ac = False
        a = False
        if self.cb_AC.GetValue():
            self.options.mSetPlot.append('AC')
            ac = True
        if self.cb_LC.GetValue():
            self.options.mSetPlot.append('LC')
            ac = True
        if self.cb_MDC.GetValue():
            self.options.mSetPlot.append('MDC')
            ac = True
        if self.cb_A.GetValue():
            self.options.mSetPlot.append('A')
            a = True
        if self.cb_LCA.GetValue():
            self.options.mSetPlot.append('LCA')
            a = True
        if self.cb_MDA.GetValue():
            self.options.mSetPlot.append('MDA')
            a = True
        if self.cb_err.GetValue():
            self.options.mSetPlot.append('E')
        if self.cb_line.GetValue():
            self.options.mSetPlot.append('L')
        if self.cb_dots.GetValue():
            self.options.mSetPlot.append('P')
        if self.cb_legend.GetValue():
            self.options.mSetPlot.append('G')
            
        if a and ac: 
            wx.MessageBox("You are plotting A and AC at the same time. Please change."
                                      ,"", wx.OK|wx.CENTRE)
            self.change = True
        self.options.mSetPlotIso = []
        if self.cb_xe133.GetValue():
            self.options.mSetPlotIso.append('Xe-133')
        if self.cb_xe131m.GetValue():
            self.options.mSetPlotIso.append('Xe-131m')
        if self.cb_xe133m.GetValue():
            self.options.mSetPlotIso.append('Xe-133m')
        if self.cb_xe135.GetValue():
            self.options.mSetPlotIso.append('Xe-135')
        if self.cb_rn222.GetValue():
            self.options.mSetPlotIso.append('Rn-222')

        try:
            with open(self.options.pathFile, 'w') as file:
                txt = [self.options.dataPath, self.options.detbkPath, self.options.resultPath]
                for line in txt:
                    file.write(line)
                    file.write("\n")
                file.close()
        except IOError:
            wx.LogError("Cannot write to pathFile %s. Data paths set to home directory.\n" %self.options.pathFile)
            self.dataPath = str(Path.home())
            self.detbkPath = str(Path.home())
            self.resultPath = str(Path.home())
        myEvent = SaveEvent()
        wx.PostEvent(self.parent, myEvent)
        event.Skip()
        
    def OnClose(self,event):
        """Perform some checks and then close the options panel"""
        if self.change:
              wx.MessageBox("You are plotting A and AC at the same time. Please change."
                                      ,"", wx.OK|wx.CENTRE)
              return
        if not path.exists(self.options.dataPath):
            wx.MessageBox("Data path does not exist. Please modify"
                                      ,"", wx.OK|wx.CENTRE)
            return
        if not path.exists(self.options.detbkPath):
            wx.MessageBox("Detector background path does not exist. Please modify"
                                      ,"", wx.OK|wx.CENTRE)
        if not path.exists(self.options.resultPath):
            wx.MessageBox("Result path does not exist. Please modify"
                                      ,"", wx.OK|wx.CENTRE)
            return
        self.Close()
        
"""
app = wx.App(False)
opt = options()
optionsPanel(None,"Options",opt).Show()
app.MainLoop()
del app
"""